/*  Alib  */

#include <string.h>
#include "../../src/mem_manager.h"


#define EXTERN_ALIB
#include "Alib.h"

#ifdef HAVE_X11_DBE
	#include "Vdbe.h"
#endif


void SetPoint(Point *p, int x, int y)
{
	p->x = x;
	p->y = y;
}


void SetRect(Rect *r, int ax, int ay, int bx, int by)
{
	if( ax >= bx || ay >= by ){
		r->a.x = 0;
		r->a.y = 0;
		r->b.x = 0;
		r->b.y = 0;
	} else {
		r->a.x = ax;
		r->a.y = ay;
		r->b.x = bx;
		r->b.y = by;
	}
}


int IsEmptyRect(Rect *r)
{
	if( (r->a.x >= r->b.x) || (r->a.y >= r->b.y) )
		return 1;
	else
		return 0;
}


void ExpandRect(Rect *r, int dx, int dy)
{
	SetRect(r, r->a.x - dx, r->a.y - dy, r->b.x + dx, r->b.y + dy);
}


static int min(int x, int y)
{
	if( x <= y )
		return x;
	else
		return y;
}


static int max(int x, int y)
{
	if( x >= y )
		return x;
	else
		return y;
}


void IntersectRect(Rect *r1, Rect *r2, Rect *res)
{
	SetRect(res,
		max(r1->a.x, r2->a.x), max(r1->a.y, r2->a.y),
		min(r1->b.x, r2->b.x), min(r1->b.y, r2->b.y));
}


/*
#define FLOAT_SLOPE
	Comment-out this macro if you want "int" precision arithmetic on
	line slopes: faster but some garbage on the screen may appear.
*/


#define COLLAPSEUNUSEDPOINTS
/*
	Another magic flag of unknown meaning to me. Another one with the same
	name appears in Vdbe.c: are the two flags related? To investigate.
	[U.S.]
*/


/*
 *  Polygons are built from edges
 */

typedef struct _Edge {
	short     y2;				/* Ending y location */
#ifdef FLOAT_SLOPE
	float     x1;				/* Starting x location */
	float     Dx;				/* Inverse slope of edge line */
#else
	long      x1;				/* Starting x location */
	long      Dx;				/* Inverse slope of edge line */
#endif
	ZInfo    *p;				/* depth and color information */
	struct _Edge *nexte;		/* next edge on this edge list */
	struct _Edge *next;			/* next edge on active edge list */
} _Edge;


/*
 *  An edge list contains a pointer to a list of Edges.
 */

typedef struct _EdgeList {
	Edge     *head;				/* pointer to first edge in list */
} _EdgeList;


/*
 *  A ScanLine is composed of a vector of ColorSegment's.
 */

typedef struct _ScanLine {
	ColorSegment *head;			/* first element in the list */
	ColorSegment *tail;			/* last element in the list */
	int       count;
} _ScanLine;


/*
 *  A ColorSegment is a row of adjacent pixels all of the same color.
 */

typedef struct _ColorSegment {
	short     x;				/* starting x location */
	unsigned short length;		/* number of pixels of this color */
	Color     color;			/* color of this segment */
} _ColorSegment;


#define MAX_CS_PER_LINE 128
/*
	NOTE. This constant is the max no. of ColorSegment per ScanLine, and
	its value depends on the complexity of the image. Increase if you get
	the "Color Segment pool overflow" error. In the worst case this value
	is the width of the window.
*/


#ifdef X11

#include <X11/Xlib.h>

#ifdef HAS_FRAME_BUFFER

static void
LocateWindowOrigin(display, w, xo, yo)
Display  *display;
Window    w;
int      *xo, *yo;
{
	XWindowAttributes win;
	Window    child;
	int       wx, wy;

	XGetWindowAttributes(display, w, &win);

	XTranslateCoordinates(display, w, win.root,
				 -win.border_width, -win.border_width, &wx, &wy, &child);

	*xo = wx;
	*yo = wy;

}

#endif

#endif


static void
awindow_alloc_internals(AWindow *w)
/*
	Allocate internal buffers based on current width,height, then
	set counters and indeces to these buffers.
*/
{
	int i;

	w->edges = MALLOC((w->height + 1) * sizeof(EdgeList));
	w->lines = MALLOC((w->height + 1) * sizeof(EdgeList));
	w->EPSize = 64 * 1024;
	w->edgePool = MALLOC(w->EPSize * sizeof(Edge));
	memset(w->edgePool, 0, w->EPSize * sizeof(Edge));

	w->scanLine = MALLOC((w->height + 1) * sizeof(ScanLine));
	w->lastScanLine = MALLOC((w->height + 1) * sizeof(ScanLine));

	if (w->doubleBuffered) {
		w->otherLastScanLine = MALLOC((w->height + 1) * sizeof(ScanLine));
	}
	else {
		w->otherLastScanLine = NULL;
	}

	w->CSSize0 = w->height * MAX_CS_PER_LINE;
	w->csPool0 = MALLOC(w->CSSize0 * sizeof(ColorSegment));
	memset(w->csPool0, 0, w->CSSize0 * sizeof(ColorSegment));

	w->CSSize1 = w->height * MAX_CS_PER_LINE;
	w->csPool1 = MALLOC(w->CSSize1 * sizeof(ColorSegment));
	memset(w->csPool1, 0, w->CSSize1 * sizeof(ColorSegment));

	if (w->doubleBuffered) {
		w->CSSize2 = w->height * MAX_CS_PER_LINE;
		w->csPool2 = MALLOC(w->CSSize2 * sizeof(ColorSegment));
		memset(w->csPool2, 0, w->CSSize2 * sizeof(ColorSegment));
	}
	else {
		w->CSSize2 = 0;
		w->csPool2 = NULL;
	}

	w->EPTop = w->CSTop0 = w->CSTop1 = w->CSTop2 = w->curPool = 0;

	w->ymin = w->height;
	w->ymax = 0;

	for (i = 0; i < w->height + 1; ++i) {

		w->scanLine[i].count = 0;
		w->scanLine[i].head = 0;
		w->scanLine[i].tail = 0;
		w->lastScanLine[i].count = 0;
		w->lastScanLine[i].head = 0;
		w->lastScanLine[i].tail = 0;
		if (w->doubleBuffered) {
			w->otherLastScanLine[i].count = 0;
			w->otherLastScanLine[i].head = 0;
			w->otherLastScanLine[i].tail = 0;
		}
		w->edges[i].head = NULL;
		w->lines[i].head = NULL;

	}
}


static void
awindow_free_internals(AWindow *w)
/*
	Release all the internal data buffers, but DO NOT set counters nor
	indices and does not free nor initialize: bseg[], zpool, aPixel.
*/
{
	FREE(w->edges);
	FREE(w->lines);
	FREE(w->edgePool);
	FREE(w->scanLine);
	FREE(w->lastScanLine);
	FREE(w->otherLastScanLine);
	FREE(w->csPool0);
	FREE(w->csPool1);
	FREE(w->csPool2);
}


/* FIXME: */
static int maxpcolors;


AWindow  *
AlibOpen(Display * display, int screen, Window win, Colormap cmap,
	Visual * visual, int width, int height)
{

	AWindow  *w;
	int       planes, i;
	int       x, y;
	Window    root;
	unsigned  int bwidth, depth;
	XGCValues gcv;

	if( width < 1 )
		width = 1;

	if( height < 1 )
		height = 1;

	w = MALLOC(sizeof(AWindow));
	memset(w, 0, sizeof(AWindow));

	w->display = display;
	w->screen = screen;
	w->win = win;
	w->cmap = cmap;
	w->visual = visual;
	w->width = width;
	w->height = height;
	SetRect(&w->rect, 0, 0, width, height);
	w->clip = w->rect;
	w->flags = VPFastAnimation; /* FIXME: any other flag here? VPDoubleBuffer? */
	w->set = 0;

	maxpcolors = (visual->map_entries == 2) ? AMaxPixelValue : visual->map_entries;
	w->aPixel = MALLOC(sizeof(Color) * maxpcolors);
	memset(w->aPixel, 0, sizeof(Color) * maxpcolors);
	for (i = 0; i < MAXCOLORS; ++i) {
		w->bPixel[i] = 0;
	}
	w->pixel = &(w->aPixel[0]);  /* FIXME: ? */

	w->colorTweakProc = NULL;

/*
 *  Is this a monochrome situation?  We also resort to monochrome on
 *  color systems that probably won't have enough color cell space
 *  to support double buffering.  We'll also use pixmaps if the
 *  global variable "usePixmaps" is non-zero.
 */

	planes = DisplayPlanes(display, screen);

	if( w->flags & VPDoubleBuffer
	|| planes < PLANECOUNT * 2 + 2
	|| _VDefaultWorkContext->usePixmaps != 0) {

		if( (w->flags & VPFastAnimation) == 0 ){
			w->monoPixmap = XCreatePixmap(display,
						 RootWindow(display, screen), width, height, planes);
			w->flags |= VPPixmap;
		}

		if( w->flags & VPDoubleBuffer ){

#ifdef HAVE_X11_DBE
			/*
				Use faster Vdbe low-level drawing routines:
			*/

			w->doubleBuffered = 1;
			w->gc = XCreateGC(display, RootWindow(display, screen), 0, NULL);
			w->flags |= VPPixmap;
			w->dbeDrawable =  XdbeAllocateBackBufferName(display, win,
						XdbeUntouched);
			w->lastPixel = 0xffffffff;


			w->AllocColor = VdbeAllocColor;
			w->SetClipRect = VdbeSetClipRect;
			w->DrawLine = VdbeDrawLine;
			w->DrawSegments = VdbeDrawSegments;
			w->FillRectangle = VdbeFillRectangle;
			w->FillPolygon = VdbeFillPolygon;
			w->ExposeBuffer = VdbeExposeBuffer;

#else
			fprintf(stderr, "X Window DBE extension not available, "
				"please disable double buffering option.\n");
			exit(1);
#endif

		} else {

			if( planes == 1 || _VDefaultWorkContext->usePixmaps != 0 )
				w->flags |= VPMono;

			w->mask = AllPlanes;
		}

#ifdef X11

	} else {


		/*
			Use slower internal low-level drawing routines:
		*/

		w->AllocColor = AlibAllocColor;
		w->SetClipRect = AlibSetClipRect;
		w->DrawLine = AlibDrawLine;
		w->DrawSegments = AlibDrawSegments;
		w->FillRectangle = AlibFillRectangle;
		w->FillPolygon = FillPolygon; /* FIXME: it should be AlibFillPolygon? */
		w->ExposeBuffer = AlibExposeBuffer;

#else

	} else {

		fprintf(stderr, "No graphic support compiled-in. "
			"Check compilation options for X Window or Windows or whatever.\n");
		exit(0);
#endif

	}

	w->colors = 0;
	w->zsize = 32768;
	w->ztop = 0;
	w->zpool = MALLOC(sizeof(ZInfo) * w->zsize);
	w->depth = MaxDepth;

	awindow_alloc_internals(w);

	w->color_to_pixel_map_top = 0;

	w->bsegSize = AMaxPixelValue + 1;

	for (i = 0; i < AMaxPixelValue + 1; ++i)
		w->bseg[i].count = -1;

	XGetGeometry(display, win, &root, &x, &y,
		(unsigned int *) &width, (unsigned int *) &height, &bwidth, &depth);
	w->d = win;
	gcv.cap_style = CapButt;
	w->gc = XCreateGC(display, RootWindow(display, screen),
					  GCCapStyle, &gcv);

	if( w->flags & AMonoMode )
		XSetFillStyle(display, w->gc, FillOpaqueStippled);

#ifdef HAS_FRAME_BUFFER
	FrameBufferIOInit();
	LocateWindowOrigin(display, d, &x, &y);
	FrameBufferSetOrigin(x, y);
#endif

	return w;
}


void
AlibResize(AWindow * w, int width, int height)
{
	if( width < 1 )
		width = 1;

	if( height < 1 )
		height = 1;

	if( w->width == width  &&  w->height == height ){
		return;
	}

	awindow_free_internals(w);

	w->width = width;
	w->height = height;
	SetRect(&w->rect, 0, 0, width, height);
	IntersectRect(&w->clip, &w->rect, &w->clip);

	awindow_alloc_internals(w);

/*
 *  Allocate a new monochrome bitmap to buffer the displayed image.
 */

	if( w->flags & VPPixmap ){
		XFreePixmap(w->display, w->monoPixmap);
		w->monoPixmap = XCreatePixmap(w->display,
			RootWindow(w->display, w->screen), width, height,
		DisplayPlanes(w->display, w->screen));
	}

}


void
AlibClose(AWindow *w)
{
	int i;

	awindow_free_internals(w);

	for (i = 0; i < AMaxPixelValue + 1; ++i){
		if( w->bseg[i].count >= 0 ){
			FREE(w->bseg[i].head);
		}
	}

	FREE(w->zpool);

	FREE(w->aPixel);

	FREE(w);
}


int AlibSetDepthCueing(AWindow *w, int flag)
{
	if( _VDefaultWorkContext->depthCueSteps > 1
	&& (w->flags & (VPFastAnimation | VPDoubleBuffer))) {
		if( flag )
			w->flags |= VPDepthCueing;
		else
			w->flags &= ~VPDepthCueing;
		return 0;
	}
	else {
		return -1;
	}
}


static ColorSegment *
CSOverflow(void)
{
	static int warning_raised = 0;

	if( ! warning_raised ){

		fprintf(stderr, "Color Segment pool overflow.\n"
			"Try increasing the constant MAX_CS_PER_LINE in V/lib/Alib.h\n");

		warning_raised = 1;
	}

	return NULL;
}


static ColorSegment *
mallocCS(AWindow * w)
{
	switch (w->curPool) {
	case 0:
		return (w->CSTop0 == w->CSSize0) ?
			CSOverflow() : &(w->csPool0[(w->CSTop0)++]);
	case 1:
		return (w->CSTop1 == w->CSSize1) ?
			CSOverflow() : &(w->csPool1[(w->CSTop1)++]);
	case 2:
		return (w->CSTop2 == w->CSSize2) ?
			CSOverflow() : &(w->csPool2[(w->CSTop2)++]);
	default:
		abort();
	}
}


static __inline void
freeLastCS(AWindow * w)
{
	switch (w->curPool) {
	case 0:
		w->CSTop0--;
		break;
	case 1:
		w->CSTop1--;
		break;
	case 2:
		w->CSTop2--;
		break;
	}
}


static void
DrawScanLine(AWindow * w, int y, int x0, int x1, Color color)
/*
	Draw a line segment ranging from pixel (x0,y) to (x1-1,y) so that the
	segment is exactly (x1-x0) pixels long. Does nothing if the length is
	zero or negative, i.e. x1-x0<=0. Successive calls on the same scan line
	must be ordered by x0 value.
*/

{

	register ColorSegment *p;
	register int lastx;

	if( y < 0 || y >= w->height ){
		printf("FIXME: DrawScanLine(): y=%d\n", y);
	}

#ifdef DEBUG
	printf("DrawScanLine (w, %d, %d, %d, %d)\n", y, x0, x1, color);
#endif

	if( x0 >= x1 )
		return;

	if (w->scanLine[y].count == 0) {

/*
 *  This is the first segment on this scan line.
 */

		p = mallocCS(w);
		if (p != NULL) {
			p->x = x0;
			p->length = x1 - x0;
			p->color = color;
			w->scanLine[y].head = w->scanLine[y].tail = p;
			w->scanLine[y].count = 1;
		}

	}

/*
 *  The scan line wasn't empty.  Check the previous scan line entry for
 *  any overlap with this one.
 */

	else {
		p = w->scanLine[y].tail;
		lastx = p->x + p->length;

		if (lastx > x0) {

/*
 *  The segments overlap.
 *
 *  If the overlapping segments have the same color, then simply
 *  extend the previous segment's length.
 */

			if (p->color == color)
				p->length = x1 - p->x;

/*
 *  The overlapping segment's colors are different.  Shorten the previous
 *  segment and allocate an entry for the current one.  If the shortened
 *  segment is eliminated, use it to store this segment's information.
 */

			else {
				p->length = x0 - p->x;
				if (p->length > 0) {
					if ((p = mallocCS(w)) != NULL) {
						++w->scanLine[y].count;
						w->scanLine[y].tail = p;
					}
				}

/*
 *  If the shortened segment's length went to zero, we may need to merge
 *  this segment with the last one.
 */

				if (((p - 1)->color == color) &&
					(w->scanLine[y].count > 1)) {
					freeLastCS(w);
					p = w->scanLine[y].tail = p - 1;
					--w->scanLine[y].count;
					p->length = x1 - p->x;
				}
				else {
					p->x = x0;
					p->length = x1 - x0;
					p->color = color;
				}
			}
		}

/*
 *  The segments do not overlap.
 *
 *  If the segments are adjacent and the colors are the same, extend the
 *  last segment.
 *
 *  Otehrwise, create a new segment and append it to this line.
 */

		else if ((lastx == x0) && (p->color == color))
			p->length = x1 - p->x;

		else if ((p = mallocCS(w)) != NULL) {
			++w->scanLine[y].count;
			w->scanLine[y].tail = p;
			p->x = x0;
			p->length = x1 - x0;
			p->color = color;
		}

	}

}


#define QDrawScanLine(w, y, x0, x1, c) \
	{                               \
		if (lastStart == -1) {      \
			lastStart = x0;         \
			lastColor = c;          \
		}                           \
		else if (c != lastColor) {  \
			DrawScanLine (w, y, lastStart, x0, lastColor);  \
			lastStart = x0;         \
			lastColor = c;          \
		}                           \
	}


static void
updateActiveEdgeList(AWindow * w, Edge ** head, Edge ** lineHead, int y)
/*
	Remove entries from the active edge table that should no longer
	be drawn.  Increment the x value of all others.  Add all entries
	that begin on this row.
*/
{

	int inserted;
	Edge *p, *q, *last, *newhead, *newtail, *q1, *q2;

/*
 *  Remove stale edges from the active list and increment x values of the rest.
 *
 *  We get a bit tricky here.  The active edge table must be ordered by x1
 *  value to allow for the plane sweep method to work.  The relative edge
 *  sequence may change as each edge's x1 value changes.  We'll build an
 *  updated, sorted "partial" edge list as we increment x1 values.  An edge that
 *  no longer belongs after its predecessor in the list is temporarily moved
 *  to this row's new edge list so that it can be inserted in it's proper
 *  position.
 */

	newhead = newtail = NULL;

	for (p = *head; p != NULL; p = p->next) {

		if (p->y2 != y) {

			p->x1 += p->Dx;

			if (newhead == NULL) {
				newhead = newtail = p;
			}
			else if (p->x1 >= newtail->x1) {
				newtail->next = p;
				newtail = p;
			}
			else {
				p->nexte = w->edges[y].head;
				w->edges[y].head = p;
			}

		}

	}

	*head = newhead;
	if (newtail != NULL)
		newtail->next = NULL;

/*
 *  Add any new entries to the active line list.
 */

	for (q = w->lines[y].head; q != NULL; q = q->nexte) {
		q->next = *lineHead;
		*lineHead = q;
	}

/*
 *  Add edges corresponding to each active line.
 */

	newhead = newtail = NULL;

	for (q = *lineHead; q != NULL; q = q->next) {
		if (y < q->y2) {
			q1 = q + 1;
			q2 = q + 2;
			q1->x1 = q->x1;
			q2->x1 = q->x1 = q->x1 + q->Dx;
			q1->y2 = q2->y2 = y + 1;
			q1->nexte = q2;
			q2->nexte = w->edges[y].head;
			w->edges[y].head = q1;
			if (newhead == NULL) {
				newhead = newtail = q;
			}
			else {
				newtail->next = q;
				newtail = q;
			}
		}
	}

	*lineHead = newhead;
	if (newtail != NULL)
		newtail->next = NULL;

/*
 *  Insert all new edges for this row in sorted order onto the active edge list
 */

	for (q = w->edges[y].head; q != NULL; q = q->nexte) {

#ifdef DEBUG
		fprintf(stderr, "adding edge at 0x%x; x = %d, y2 = %d\n", q,
				q->x1 >> 16, q->y2);
#endif

		if (*head == NULL) {
			*head = q;
			q->next = NULL;
		}
		else {
			inserted = 0;
			for (p = *head, last = NULL; p != NULL;) {
				if (q->x1 <= p->x1) {
					if (last == NULL)
						*head = q;
					else
						last->next = q;
					q->next = p;
					inserted = 1;
					break;
				}
				last = p;
				p = p->next;
			}
			if (inserted == 0) {
				last->next = q;
				q->next = NULL;
			}
		}

	}

#ifdef DEBUG
	fprintf(stderr, "Active Edge Table:\n");
	for (p = *head; p != NULL; p = p->next) {
		fprintf(stderr, "    x = %d, y2 = %d, Dx = 0x%u, depth = %u\
, color = %d\n", p->x1 >> 16, p->y2, p->Dx, p->p->depth, p->p->color);
	}
#endif

}


#define RIGHT_SIDE_CORRECTION

static void
planeSweep(AWindow * w, Edge * head, int y)
/*
	Draw all ploygons on this scan line by the plane sweep method.
*/
{

	ZInfo    *ps = NULL;	/* polygon set head */
	ZInfo    *q, *r, *lastr = NULL;
	Edge     *p;				/* current edge */
	register int x0, x1 = 0, lastStart = -1;
	Color     lastColor = 0;
	register unsigned long lastDepth = MaxDepth;

	if (head == NULL) {
		w->scanLine[y].count = 0;
		return;
	}

	for (p = head; p->next != NULL; p = p->next) {

#ifdef FLOAT_SLOPE
		x0 = p->x1;
		x1 = p->next->x1;
#else
		x0 = p->x1 >> 16;
		x1 = p->next->x1 >> 16;
#endif

/*
 *  Polygons are ordered on the ps list by depth.  We use a special flag
 *  to determine quickly if the current polygon is or is not an element of the
 *  currently active polygon set.  Since the polygon set is ordered by depth,
 *  the first element determines the color that's drawn.
 */

		q = p->p;
		if (q->next == NotAnElement) {
			if (ps == NULL) {
				ps = q;
				q->next = q->prev = NULL;
			}
			else {
				for (r = ps; r != NULL; r = r->next) {
					if (q->depth < r->depth) {
						if (r->prev == NULL)
							ps = q;
						else
							r->prev->next = q;
						q->next = r;
						q->prev = r->prev;
						r->prev = q;
						break;
					}
					lastr = r;
				}
				if (r == NULL) {
					q->next = lastr->next;
					lastr->next = q;
					q->prev = lastr;
				}
			}

		}
		else {
			if (q->prev == NULL)
				ps = q->next;
			else
				q->prev->next = q->next;
			if (q->next != NULL)
				q->next->prev = q->prev;
			q->next = NotAnElement;
		}

/*
 *  If the polygon set is non-null, then there is some line segment that
 *  should be plotted.  We'll perform one small correction here:  if the depth
 *  of the last adjacent polygon segment was less than the depth of this one,
 *  then add one to the x0 value.  This prevents the right side of a polygon
 *  from looking different than the left side.
 */

#ifdef RIGHT_SIDE_CORRECTION
		if (ps != NULL) {
			if (lastDepth < ps->depth) {
				if ((++x0) <= x1) {
					QDrawScanLine(w, y, x0, x1, ps->color);
					lastDepth = ps->depth;
				}
			}
			else {
				QDrawScanLine(w, y, x0, x1, ps->color);
				lastDepth = ps->depth;
			}
		}
		else {
			lastDepth = MaxDepth;
			if (lastStart != -1) {
				DrawScanLine(w, y, lastStart, x1, lastColor);
				lastStart = -1;
			}
		}
#else
		if (ps != NULL)
			DrawScanLine(w, y, x0, x1, ps->color);
#endif

	}

	if (lastStart != -1)
		DrawScanLine(w, y, lastStart, x1, lastColor);

	if (ps != NULL) {
#ifdef DEBUG
		if (ps->next != NULL)
			fprintf(stderr, "More then 1 element left at end of planeSweep\n");
#endif
		ps->next = NotAnElement;
	}

}


static void
EdgeTableToScanLine(AWindow * w)
/*
	Convert the edge table to scan line segments.
*/
{

	int       y;
	Edge     *active;			/* head of active edge list */
	Edge     *activeLines;		/* head of active line list */

	active = activeLines = NULL;

	//if (w->ymin < 0 || w->ymax >= w->height) {
	if (w->ymin < 0 || w->ymax > w->height) {  /* FIXME: fixes bug in updateActiveEdgeList() */
		fprintf(stderr, "FIXME: the y bounds are out of line.\n");
		fprintf(stderr, "ymin = %d   ymax = %d\n", w->ymin, w->ymax);
		fprintf(stderr, "height = %d\n", w->height);
		abort();
	}

	for (y = w->ymin; y <= w->ymax; ++y) {

		updateActiveEdgeList(w, &active, &activeLines, y);

		planeSweep(w, active, y);

	}
}



#ifdef X11

#ifdef X11_DRAW_SEGMENTS

static void
AX11AllocBufferedSegments(AWindow * w, Color c)
{

#ifdef X11_DRAW_SEGMENTS
	w->bseg[c].head = MALLOC(w->bsegSize * sizeof(XSegment));
#else
	w->bseg[c].head = MALLOC(w->bsegSize * sizeof(XRectangle));
#endif
	w->bseg[c].count = 0;

}


static void /* FIXME: restore "inline" flag */
OutputSegment(AWindow *w, int yp, int x0p, int x1p, Color c)
{
	int		i;
	XSegment	*p;

	if( c >= w->bsegSize ){
		printf("FIXME: c=%d out of [0,%d]\n", c, w->bsegSize-1);
		c = 0;
	}
	if (w->bseg[c].count == -1)
		AX11AllocBufferedSegments (w, c);
	i = ++(w->bseg[c].count);
	p = &(w->bseg[c].head[i-1]);
	p->x1 = x0p;
	p->x2 = x1p;
	p->y1 = p->y2 = yp;
	if (i == w->bsegSize) {
		if (w->flags & AMonoMode)
			XSetStipple (w->display, w->gc, w->stipple[c]);
		else
			XSetForeground (w->display, w->gc, w->color_to_pixel_map[c]);
		p = w->bseg[c].head;
		XDrawSegments (w->display, w->d, w->gc, p, w->bsegSize);
		w->bseg[c].count = 0;
	}
}

#else

static inline void
OutputSegment(AWindow *w, int yp, int x0p, int x1p, Color c)
{
	register int		i;
	register XRectangle	*p;

	if (w->bseg[c].count == -1)
		AX11AllocBufferedSegments (w, c);
	i = ++(w->bseg[c].count);
	p = &(w->bseg[c].head[i-1]);
	p->x = x0p;
	p->width = x1p - x0p + 1;
	p->y= yp;
	p->height = 1;
	if (i == w->bsegSize) {
		if (w->flags & AMonoMode)
			XSetStipple (w->display, w->gc, w->stipple[c]);
		else
			XSetForeground (w->display, w->gc, w->color_to_pixel_map[c]);
		p = w->bseg[c].head;
		XFillRectangles (w->display, w->d, w->gc, p,
			w->bsegSize);
		w->bseg[c].count = 0;
	}
}

#endif

static void
AX11FlushBufferedSegments(AWindow * w)
{

	register int i;

	for (i = 0; i < AMaxPixelValue; ++i) {
		if (w->bseg[i].count > 0) {
			if (w->flags & AMonoMode) {
				XSetStipple(w->display, w->gc, w->stipple[i]);
			}
			else {
				XSetForeground(w->display, w->gc, w->color_to_pixel_map[i]);
			}
#ifdef X11_DRAW_SEGMENTS
			XDrawSegments(w->display, w->d, w->gc,
						  w->bseg[i].head, w->bseg[i].count);
#else
			XFillRectangles(w->display, w->d, w->gc,
							w->bseg[i].head, w->bseg[i].count);
#endif
			w->bseg[i].count = 0;
		}
	}
}

#endif

#ifdef DEBUG
static void
printScanLine(s, count)
ColorSegment *s;
int       count;
{

	for (; count > 0; --count, ++s)
		printf("[%d %d %d ] ", s->x, s->length, s->color);

	printf("\n");
}
#endif


static void
ScanLineDifference(AWindow * w)
/*
	Determine the differences between this frame and the previous one
	and plot the differences.
*/
{

	register ColorSegment *cur, *last;
	register int len, curX = 0, curLength = 0, curCount, lastCount, y;

	for (y = 0; y < w->height; ++y) {

		cur = w->scanLine[y].head;
		curCount = w->scanLine[y].count;

		if (curCount > 0) {
			curX = cur->x;
			curLength = cur->length;
		}

		last = w->lastScanLine[y].head;
		lastCount = w->lastScanLine[y].count;

#ifdef DEBUG
		if (curCount > 1 || lastCount > 1) {
			printf("\ny = %d\nLast scan line:\n", y);
			printScanLine(last, lastCount);
			printf("Current scan line:\n");
			printScanLine(cur, curCount);
		}
#endif

		while (curCount != 0) {

			if (lastCount == 0) {
				if (cur->color != UnchangedColor) {
#ifdef DEBUG
					printf("row: %d, %d to %d color %d\n",
						   y, curX, curX + curLength - 1,
						   cur->color);
#endif
#ifdef HAS_FRAME_BUFFER
					FrameBufferRun(curX, y, curLength,
								   cur->color);
#else
					OutputSegment(w, y, curX, curX + curLength - 1,
								  cur->color);
#endif
				}
				++cur;
				curX = cur->x;
				curLength = cur->length;
				--curCount;
			}

			else if (curX == last->x) {

				if (curLength == last->length) {

/*
 *  The trivial case:  Both segments have the same length.  If the colors are
 *  different, then output the correct new color.
 */

					if (cur->color != last->color) {
						if (cur->color != UnchangedColor) {
#ifdef HAS_FRAME_BUFFER
							FrameBufferRun(curX, y, curLength,
										   cur->color);
#else
							OutputSegment(w, y, curX,
										  curX + curLength - 1,
										  cur->color);
#endif
#ifdef DEBUG
							printf("row: %d, %d to %d color %d\n",
								   y, curX,
								   curX + curLength - 1,
								   cur->color);
#endif
						}
					}
					++last;
					++cur;
					curX = cur->x;
					curLength = cur->length;
					--curCount;
					--lastCount;
				}

				else if (curLength < last->length) {

/*
 *  The current segment is shorter than the last one.  Output a new line segment
 *  if the colors differ; delete the current segment and update the last one's
 *  x value and length.
 */
					if (cur->color != last->color) {
						if (cur->color != UnchangedColor) {
#ifdef HAS_FRAME_BUFFER
							FrameBufferRun(curX, y, curLength,
										   cur->color);
#else
							OutputSegment(w, y, curX,
										  curX + curLength - 1,
										  cur->color);
#endif
#ifdef DEBUG
							printf("row: %d, %d to %d color %d\n",
								   y, curX,
								   curX + curLength - 1,
								   cur->color);
#endif
						}
					}
					last->x += curLength;
					last->length -= curLength;
					++cur;
					curX = cur->x;
					curLength = cur->length;
					--curCount;
				}

/*
 *  The current segment is longer than the last one.  Output, if necessary;
 *  delete the last segment and update the current segment's x value and length.
 */

				else {
					if (cur->color != last->color) {
						if (cur->color != UnchangedColor) {
#ifdef HAS_FRAME_BUFFER
							FrameBufferRun(curX, y, last->length,
										   cur->color);
#else
							OutputSegment(w, y, curX,
										  curX + last->length - 1,
										  cur->color);
#endif
#ifdef DEBUG
							printf("row: %d, %d to %d color %d\n",
								   y, curX,
								   curX + lastLength - 1,
								   cur->color);
#endif
						}
					}
					curX += last->length;
					curLength -= last->length;
					++last;
					--lastCount;
				}

			}

/*
 *  The current segment starts before any previous segments.
 */

			else if (curX < last->x) {
				if (curX + curLength > last->x)
					len = last->x - curX;
				else
					len = curLength;

				if (cur->color != UnchangedColor) {
#ifdef HAS_FRAME_BUFFER
					FrameBufferRun(curX, y, len,
								   cur->color);
#else
					OutputSegment(w, y, curX, curX + len - 1,
								  cur->color);
#endif
#ifdef DEBUG
					printf("row: %d, %d to %d color %d\n",
						   y, curX, curX + len - 1, cur->color);
#endif
				}
				curX += len;
				curLength -= len;
				if (curLength == 0) {
					++cur;
					curX = cur->x;
					curLength = cur->length;
					--curCount;
				}
			}

/*
 *  The previous segment begins before the first current segment.
 */

			else {
				if (last->x + last->length > curX)
					len = curX - last->x;
				else
					len = last->length;

				last->x += len;
				last->length -= len;
				if (last->length == 0) {
					++last;
					--lastCount;
				}
			}
		}
	}							/* for (y=... */
}


/*
 *  Randomly changing drawing colors in X would require that we add an XChangeGC
 *  request before virtually every line segment draw.
 *  Once the scan line segment output has been optimized by ScanLineDifference,
 *  we can add a further optimization to reduce the number of XChangeGC
 *  requests that are required.  We'll add a structure to group drawing
 *  requests by color.  When some watermark is past, an XChangeCG request may
 *  be issued, followed by all requests for that color.  This request buffer
 *  will be flushed with a call to FrameComplete().
 */

static void
FrameComplete(AWindow * w)
{

	register ScanLine *tmp;
	register int i;

#ifdef DEBUG
	register int cstop;

#endif

/*
 *  Convert the edge table to scan line segments.
 */

	EdgeTableToScanLine(w);

/*
 *  Determine the differences between this frame and the previous one
 *  and plot the differences.
 */

	ScanLineDifference(w);

/*
 *  This frame now becomes the previous frame; clear the new current frame.
 */

	tmp = w->lastScanLine;
	if (w->doubleBuffered) {
		w->lastScanLine = w->otherLastScanLine;
		w->otherLastScanLine = w->scanLine;
	}
	else {
		w->lastScanLine = w->scanLine;
	}
	w->scanLine = tmp;

	for (i = 0; i < (w->height + 1); ++i) {
		w->scanLine[i].count = 0;
		w->edges[i].head = NULL;
		w->lines[i].head = NULL;
	}

/*
 *  Release the allocated color segments for what was the last frame.
 */

	if (w->curPool == 0) {
		w->curPool = 1;
#ifdef DEBUG
		cstop = w->CSTop1;
#endif
		w->CSTop1 = 0;
	}
	else if (w->curPool == 1) {
		if (w->doubleBuffered) {
			w->curPool = 2;
#ifdef DEBUG
			cstop = w->CSTop2;
#endif
			w->CSTop2 = 0;
		}
		else {
			w->curPool = 0;
#ifdef DEBUG
			cstop = w->CSTop0;
#endif
			w->CSTop0 = 0;
		}
	}
	else {
		w->curPool = 0;
#ifdef DEBUG
		cstop = w->CSTop0;
#endif
		w->CSTop0 = 0;
	}

/*
 *  Release the allocated elements of the edge pool.
 */

#ifdef DEBUG
	printf("color segments = %d; edges = %d\n", cstop, w->EPTop);
#endif

	w->EPTop = 0;

	w->ymin = w->height;
	w->ymax = 0;

/*
 *  Perform any graphics-dependent finish-up
 */

#ifdef X11
	AX11FlushBufferedSegments(w);
#endif

#ifdef DEBUG
	printf("End of Frame\n");
#endif

}


void
ForceWindowRedraw(AWindow * w)
{
	int i;

	for (i = 0; i < w->height; ++i) {
		w->lastScanLine[i].count = 0;
		if (w->doubleBuffered) {
			w->otherLastScanLine[i].count = 0;
		}
	}

#if defined(HAS_FRAME_BUFFER) && defined(X11)
	LocateWindowOrigin(w->display, w->d, &x, &y);
	FrameBufferSetOrigin(x, y);
#endif
}


void
AlibDrawSegments(AWindow * w, Segment * seg, int nseg, Color color)
{
	ZInfo *z;

	if (w->ztop == w->zsize) {
		printf("Z-information pool overflow\n");
		return;
	}

	z = &(w->zpool[(w->ztop)++]);
	z->depth = --w->depth;
	z->color = color;

	for (; nseg > 0; --nseg) {
		DrawLine(w, seg->x1, seg->y1, seg->x2, seg->y2, z);
		++seg;
	}
}


void
AlibDrawLine(AWindow *w, int x1, int y1, int x2, int y2, Color color)
{
	ZInfo *z;

	if (w->ztop == w->zsize) {
		printf("Z-information pool overflow\n");
		abort();
	}

	z = &(w->zpool[(w->ztop)++]);
	z->depth = --w->depth;
	z->color = color;

	DrawLine(w, x1, y1, x2, y2, z);
}


#define TICKtoRAD(a)	(a * M_PI / (180.0 * 64.0))
#define INCR		(30 * 64)

void AlibDrawArc(AWindow *w, int x, int y, int width, int height,
	int angle1, int angle2, Color color)
{
	/*
		FIXME: needs optimization avoiding to calculate all that cos/sin.
		FIXME: the no. of sides should depend on the radius.
	*/

	ZInfo *z;
	double W, H, xc, yc;
	int incr, x1, x2, y1, y2;

	if (w->ztop == w->zsize) {
		fprintf(stderr, "Z-information pool overflow\n");
		return;
	}

	z = &(w->zpool[(w->ztop)++]);
	z->depth = --w->depth;
	z->color = color;

	W = width / 2.0;
	H = height / 2.0;
	xc = x + width / 2.0;
	yc = y + height / 2.0;

	if (angle2 < 0) {
		incr = -INCR;
		angle2 = -angle2;
	}
	else
		incr = INCR;

	if (angle2 > 360 * 64)
		angle2 = 360 * 64;

	x1 = (int) (xc + (W * cos(TICKtoRAD(angle1))));
	y1 = (int) (yc - (H * sin(TICKtoRAD(angle1))));

	while (angle2 != 0) {
		angle1 += incr;
		angle2 -= INCR;
		if (angle2 < 0) {
			angle1 -= angle2;
			angle2 = 0;
		}
		x2 = (int) (xc + (W * cos(TICKtoRAD(angle1))));
		y2 = (int) (yc - (H * sin(TICKtoRAD(angle1))));
		w->DrawLine(w, x1, y1, x2, y2, z->color);
		x1 = x2;
		y1 = y2;
	}
}


void
AlibDrawRect(AWindow *w, Rect *r, Color color)
{
	Segment seg[4];

	if( IsEmptyRect(r) )
		return;

	seg[0].x1 = r->a.x;
	seg[0].y1 = r->a.y;
	seg[0].x2 = r->b.x - 1;
	seg[0].y2 = r->a.y;

	seg[1].x1 = r->b.x - 1;
	seg[1].y1 = r->a.y;
	seg[1].x2 = r->b.x - 1;
	seg[1].y2 = r->b.y - 1;

	seg[2].x1 = r->b.x - 1;
	seg[2].y1 = r->b.y - 1;
	seg[2].x2 = r->a.x;
	seg[2].y2 = r->b.y - 1;

	seg[3].x1 = r->a.x;
	seg[3].y1 = r->b.y - 1;
	seg[3].x2 = r->a.x;
	seg[3].y2 = r->a.y;

	AlibDrawSegments(w, seg, 4, color);
	/* FIXME: should call w->DrawRect once available */
}


void
AlibFillRectangle(AWindow *w, Rect *r, Color color)
{
	ZInfo *z;

	if( IsEmptyRect(r) )
		return;

	if (w->ztop == w->zsize) {
		printf("Z-information pool overflow\n");
		abort();
	}

	z = &(w->zpool[(w->ztop)++]);
	z->depth = --w->depth;
	z->color = color;

	FillRectangle(w, r, z);
}


void
AlibFillPolygon(AWindow *w, Point * pts, int npts, Color color)
{
	ZInfo *z;

	if (w->ztop == w->zsize) {
		printf("Z-information pool overflow\n");
		abort();
	}

	z = &(w->zpool[(w->ztop)++]);
	z->depth = --w->depth;
	z->color = color;

	w->FillPolygon(w, pts, npts, z);
}


void
AlibSetClipRect(AWindow *w, Rect *r)
{
	IntersectRect(&w->rect, r, &w->clip);
}


static void
MakeET(AWindow * w, Point * pts, int npts, ZInfo * zinfo)
/*
	Build an edge table based on edge coherence as described by
	Rod Salmon and Mel Slater, 1987.
*/
{

	int i, t, addedEdge = 0;
	int x1, y1, x2, y2;
	int ymin, ymax;
	Edge *e;

	ymin = w->height;
	ymax = 0;

	zinfo->next = NotAnElement;

/*
 *  Process each edge in this polygon
 */

	for (i = 1; i <= npts; ++i) {

/*
 *  Determine the end-points of this edge (x1, y1), (x2, y2).
 */

		if (i == npts) {
			x2 = pts[0].x;
			y2 = pts[0].y;
		}
		else {
			x2 = pts[i].x;
			y2 = pts[i].y;
		}
		x1 = pts[i - 1].x;
		y1 = pts[i - 1].y;

		if (y1 > y2) {

			t = x1;
			x1 = x2;
			x2 = t;
			t = y1;
			y1 = y2;
			y2 = t;

		}

		if (y1 != y2) {

			if (w->EPTop == w->EPSize) {
				fprintf(stderr, "Edge Pool Overflow\n");
				return;
			}

			e = &(w->edgePool[(w->EPTop)++]);

			e->y2 = y2;
#ifdef FLOAT_SLOPE
			e->x1 = x1;
			e->Dx = (x2 - x1) / (double) (y2 - y1);
#else
			e->x1 = (x1 << 16) + 0x8000;
			e->Dx = ((x2 - x1) << 16) / (y2 - y1);
#endif

			e->p = zinfo;
			e->nexte = w->edges[y1].head;
			w->edges[y1].head = e;

			addedEdge = 1;

			if (y1 < ymin)
				ymin = y1;

			if (y2 > ymax)
				ymax = y2;
		}

	}

	if (addedEdge) {

		if (w->ymin > ymin)
			w->ymin = (short) ymin;

		if (w->ymax < ymax)
			w->ymax = (short) ymax;
	}
}


void
DrawPoint(AWindow * w, int x, int y, ZInfo * zinfo)
{

	Edge *e, *e1;

	if (x < w->clip.a.x || x >= w->clip.b.x
	|| y < w->clip.a.y || y >= w->clip.b.y)
		return;

	zinfo->next = NotAnElement;

	if (w->EPTop + 2 > w->EPSize) {
		fprintf(stderr, "Edge Pool Overflow\n");
		return;
	}

	e = &(w->edgePool[(w->EPTop)++]);
	e1 = &(w->edgePool[(w->EPTop)++]);

	e->y2 = e1->y2 = y + 1;
#ifdef FLOAT_SLOPE
	e->x1 = e1->x1 = x;
	e->Dx = 0.0;
#else
	e->x1 = e1->x1 = (x << 16) + 0x8000;
	e->Dx = 0;
#endif
	e->p = e1->p = zinfo;

	e1->nexte = w->edges[y].head;
	e->nexte = e1;
	w->edges[y].head = e;

	if (y < w->ymin)
		w->ymin = y;

	if (y > w->ymax)
		w->ymax = y;
}


void
AlibExposeBuffer(AWindow *w)
{
	if (w->flags & VPFastAnimation) {
		w->ztop = 0;
		w->depth = MaxDepth;
		FrameComplete(w);
		XFlush(w->display);
		return;
	}

	/*
	* this is one area where more-than-one window per display gets hairy
	* -- this call really exposes the drawings buffered in all windows
	* associated with a given display.  the calling sequence needs to be
	* changed to reflect this.
	*/

	if (w->flags & VPMono) {
		XCopyArea(w->display, w->monoPixmap, w->win, w->gc,
			0, 0, w->width, w->height, 0, 0);
		XSetForeground(w->display, w->gc, WhitePixel(w->display, w->screen));
		XFillRectangle(w->display, w->monoPixmap, w->gc,
			0, 0, w->width, w->height);
		XSetForeground(w->display, w->gc, BlackPixel(w->display, w->screen));
	}
	else if (w->flags & VPPixmap) {
		XCopyArea(w->display, w->monoPixmap, w->win, w->gc,
			0, 0, w->width, w->height, 0, 0);
	}
	else if (w->set == 0) {
		w->set = 1;
		XStoreColors(w->display, w->cmap, w->aColor, w->colors * w->colors);
		w->pixel = &(w->bPixel[0]);
		XSetPlaneMask(w->display, w->gc, w->bMask);
		w->mask = w->bMask;
		/*
		for ((p=VColorList, q = &(w->bPixel[1])); p != (VColor *) 0; ++q) {
			p->xcolor.pixel = *q;
			p = p->next;
		}
		*/
	}
	else {
		w->set = 0;
		XStoreColors(w->display, w->cmap, w->bColor, w->colors * w->colors);
		w->pixel = &(w->aPixel[0]);
		XSetPlaneMask(w->display, w->gc, w->aMask);
		w->mask = w->aMask;
		/*
		for ((p=VColorList, q = &(w->aPixel[1])); p != (VColor *) 0; ++q) {
			p->xcolor.pixel = *q;
			p = p->next;
		}
		*/
	}

	XFlush(w->display);
}


Status
AlibAllocColor(AWindow *w, Colormap colormap, XColor *c)
{
	Status result;
	result = XAllocColor (w->display, colormap, c);
	if (result) {
#ifdef X11
		if( w->color_to_pixel_map_top >= AMaxPixelValue ){
			return 1;
		} else {
			w->color_to_pixel_map[w->color_to_pixel_map_top] = c->pixel;
			c->pixel = w->color_to_pixel_map_top;
			w->color_to_pixel_map_top++;
		}
#endif
	}
	return result;
}


/* FIXME: remove static limit */
#define MAX_VERT 1000

void FillPolygon(AWindow * w, Point * pts, int npts, ZInfo * zinfo)
{
	int x1, y1, x2, y2, i, np, nr;
	Point *a, *b, q;
	int a_in, b_in;
	double t;
	Point p[MAX_VERT], r[MAX_VERT];

	if( npts < 3 ){
		return;
	}

	if( npts > MAX_VERT ){
		printf("FIXME: too many vertices in FillPolygon()\n");
		abort();
	}

	x1 = w->clip.a.x;
	y1 = w->clip.a.y;
	x2 = w->clip.b.x - 1;
	//y2 = w->clip.b.y - 1;
	y2 = w->clip.b.y;  /* FIXME: fixes bug in updateActiveEdgeList() */

	/*
		Clipping of pts[0..npts-1] against the right side.
		Generating p[0..np-1].
	*/

	np = 0;
	a = &pts[npts-1];
	a_in = (a->x <= x2);
	for( i=0; i < npts ; i++ ){
		b = &pts[i];
		b_in = (b->x <= x2);
		if( a_in ){
			if( b_in ){
				p[np++] = *b;
				a = b;
				a_in = 1;
			} else {
				q.x = x2;
				t = (x2 - a->x) / (double)(b->x - a->x);
				q.y = a->y + t*(b->y - a->y);
				p[np++] = q;
				a = b;
				a_in = 0;
			}
		} else {
			if( b_in ){
				q.x = x2;
				t = (x2 - a->x) / (double)(b->x - a->x);
				q.y = a->y + t*(b->y - a->y);
				p[np++] = q;
				p[np++] = *b;
				a = b;
				a_in = 1;
			} else {
				a = b;
				a_in = 0;
			}
		}
	}

	if( np < 3 )
		return;

	/*
		Clipping of p[0..np-1] against the left side.
		Generating r[0..nr-1].
	*/

	nr = 0;
	a = &p[np-1];
	a_in = (a->x >= x1);
	for( i=0; i < np ; i++ ){
		b = &p[i];
		b_in = (b->x >= x1);
		if( a_in ){
			if( b_in ){
				r[nr++] = *b;
				a = b;
				a_in = 1;
			} else {
				q.x = x1;
				t = (x1 - a->x) / (double)(b->x - a->x);
				q.y = a->y + t*(b->y - a->y);
				r[nr++] = q;
				a = b;
				a_in = 0;
			}
		} else {
			if( b_in ){
				q.x = x1;
				t = (x1 - a->x) / (double)(b->x - a->x);
				q.y = a->y + t*(b->y - a->y);
				r[nr++] = q;
				r[nr++] = *b;
				a = b;
				a_in = 1;
			} else {
				a = b;
				a_in = 0;
			}
		}
	}

	if( nr < 3 )
		return;

	/*
		Clipping of r[0..nr-1] against the bottom side.
		Generating p[0..np-1].
	*/

	np = 0;
	a = &r[nr-1];
	a_in = (a->y <= y2);
	for( i=0; i < nr ; i++ ){
		b = &r[i];
		b_in = (b->y <= y2);
		if( a_in ){
			if( b_in ){
				p[np++] = *b;
				a = b;
				a_in = 1;
			} else {
				t = (y2 - a->y) / (double)(b->y - a->y);
				q.x = a->x + t*(b->x - a->x);
				q.y = y2;
				p[np++] = q;
				a = b;
				a_in = 0;
			}
		} else {
			if( b_in ){
				t = (y2 - a->y) / (double)(b->y - a->y);
				q.x = a->x + t*(b->x - a->x);
				q.y = y2;
				p[np++] = q;
				p[np++] = *b;
				a = b;
				a_in = 1;
			} else {
				a = b;
				a_in = 0;
			}
		}
	}

	if( np < 3 )
		return;

	/*
		Clipping of p[0..np-1] against the top side.
		Generating r[0..nr-1].
	*/

	nr = 0;
	a = &p[np-1];
	a_in = (a->y >= y1);
	for( i=0; i < np ; i++ ){
		b = &p[i];
		b_in = (b->y >= y1);
		if( a_in ){
			if( b_in ){
				r[nr++] = *b;
				a = b;
				a_in = 1;
			} else {
				t = (y1 - a->y) / (double)(b->y - a->y);
				q.x = a->x + t*(b->x - a->x);
				q.y = y1;
				r[nr++] = q;
				a = b;
				a_in = 0;
			}
		} else {
			if( b_in ){
				t = (y1 - a->y) / (double)(b->y - a->y);
				q.x = a->x + t*(b->x - a->x);
				q.y = y1;
				r[nr++] = q;
				r[nr++] = *b;
				a = b;
				a_in = 1;
			} else {
				a = b;
				a_in = 0;
			}
		}
	}

	if( nr < 3 )
		return;

	MakeET(w, r, nr, zinfo);
}


void
FillRectangle(AWindow *w, Rect *r, ZInfo * zinfo)
{
	Point     pts[4];
	Rect f;

	IntersectRect(r, &w->clip, &f);

	if( IsEmptyRect(&f) )
		return;

	pts[0] = f.a;
	pts[1].x = f.b.x - 1;
	pts[1].y = f.a.y;
	pts[2].x = f.b.x - 1;
	//pts[2].y = f.b.y - 1;
	pts[2].y = f.b.y;  /* FIXME: fixes bug in updateActiveEdgeList() */
	pts[3].x = f.a.x;
	//pts[3].y = f.b.y - 1;
	pts[3].y = f.b.y;  /* FIXME: fixes bug in updateActiveEdgeList() */

	MakeET(w, pts, 4, zinfo);
}


typedef int outcode_t;

#define NONE	0
#define Top    1
#define Bottom 2
#define Left   8
#define Right  4

static outcode_t
CompOutCode(AWindow * w, int x, int y)
{

	/*
                         
		+-----------------------------+-> x
		|                             |
		|             Bottom          |
		|      ....................   |
		|      .                  .R  |
		|     L.     w->clip      .i  |
		|     e.                  .g  |
		|     f.     rectangle    .h  |
		|     t.                  .t  |
		|      ....................   |
		|              Top            |
		+-----------------------------+
		|                              (w->width,w->height)
		v

		y
	
		Please note that is the point lies exactly on the Right border
		or the Top border it is considered outside the clipping
		rectangle.

	*/

	outcode_t code = NONE;

	if (y >= w->clip.b.y) {
		code |= Top;
	}
	else if (y < w->clip.a.y) {
		code |= Bottom;
	}

	if (x >= w->clip.b.x) {
		code |= Right;
	}
	else if (x < w->clip.a.x) {
		code |= Left;
	}
	return code;
}


static int
ClipLine(AWindow * w, int *x0, int *y0, int *x1, int *y1)
/*
	Perform the clipping of the segment (x0,y0)-(x1,y1) vs. the w->clip
	rectangle. Return value:

		0 = the segment is visible, possibly partially; in this case
		    the returned points are both inside w->clip, i.e.
			w->clip.a.x <= x0,x1 < w->clip.b.x
			w->clip.a.y <= y0,y1 < w->clip.b.y
	
		1 = segment not visible at all

	Cohen/Sutherland 2D clipping algorithm as described by Foley, van Dam,
	Feiner and Hughes.
*/
{

	outcode_t outcode0, outcode1, outcodeOut;

	outcode0 = CompOutCode(w, *x0, *y0);
	outcode1 = CompOutCode(w, *x1, *y1);

	do {
		if (outcode0 == 0 && outcode1 == 0) {
			/* Points are both inside the clip rect */
			return 0;
		}
		else if (outcode0 & outcode1) {
			/* Points are both above some side of the clip rect,
			   for ex. both above the Left side */
			return 1;
		}
		else {
			/* At least one point is outside of the clip area */

			int       x, y;

			outcodeOut = outcode0 ? outcode0 : outcode1;

			if (outcodeOut & Top) {
				x = *x0 + (*x1 - *x0) * (w->clip.b.y - 1 - *y0) / (*y1 - *y0);
				y = w->clip.b.y - 1;
			}
			else if (outcodeOut & Bottom) {
				x = *x0 + (*x1 - *x0) * (w->clip.a.y - *y0) / (*y1 - *y0);
				y = w->clip.a.y;
			}
			else if (outcodeOut & Right) {
				y = *y0 + (*y1 - *y0) * (w->clip.b.x - 1 - *x0) / (*x1 - *x0);
				x = w->clip.b.x - 1;
			}
			else {				/* Left */
				y = *y0 + (*y1 - *y0) * (w->clip.a.x - *x0) / (*x1 - *x0);
				x = w->clip.a.x;
			}

			if (outcodeOut == outcode0) {
				*x0 = x;
				*y0 = y;
				outcode0 = CompOutCode(w, *x0, *y0);
			}
			else {
				*x1 = x;
				*y1 = y;
				outcode1 = CompOutCode(w, *x1, *y1);
			}
		}

	} while (1);
}


void
DrawLine(AWindow * w, int x1, int y1, int x2, int y2, ZInfo * zinfo)
{
	int t;
	Edge *e, *e1;

	zinfo->next = NotAnElement;

	if (ClipLine(w, &x1, &y1, &x2, &y2)){
		return;
	}
	
	if (y1 > y2) {
		t = x1;
		x1 = x2;
		x2 = t;
		t = y1;
		y1 = y2;
		y2 = t;
	}

	if ((x1 == x2) || (y1 == y2)) {

		if (w->EPTop + 2 > w->EPSize) {
			fprintf(stderr, "Edge Pool Overflow\n");
			return;
		}

		e = &(w->edgePool[(w->EPTop)++]);
		e1 = &(w->edgePool[(w->EPTop)++]);

		e->y2 = e1->y2 = y2 + 1;

#ifdef FLOAT_SLOPE
		if (x1 == x2)
			e->x1 = e1->x1 = x1;
		else { /* y1 == y2 */
			e->x1 = x1;
			e1->x1 = x2;
		}
		e->Dx = e1->Dx = 0.0;
#else
		if (x1 == x2)
			e->x1 = e1->x1 = (x1 << 16) + 0x8000;
		else { /* y1 == y2 */
			e->x1 = (x1 << 16) + 0x8000;
			e1->x1 = (x2 << 16) + 0x8000;
		}
		e->Dx = e1->Dx = 0;
#endif
		e->p = e1->p = zinfo;

		e1->nexte = w->edges[y1].head;
		e->nexte = e1;
		w->edges[y1].head = e;
	}

/*
 *  A non-trivial case.  The is a slope that is non-zero and not infinity.
 *  We will need one edge entry to hang off the line list at the starting y
 *  value of the line.  We will also pre-allocate two edge entries to be used
 *  as the line'e edges during the EdgeTableToScanLine procedure.  By our own
 *  convention, we'll place them in the two edge pool entries immediately
 *  following the line information entry. 
 */

	else {

		if (w->EPTop + 3 >= w->EPSize) {
			fprintf(stderr, "Edge Pool Overflow\n");
			return;
		}

		e = &(w->edgePool[(w->EPTop)++]);
		w->EPTop += 2;  /* required -- see comment above */

		e->y2 = y2;

#ifdef FLOAT_SLOPE
		e->x1 = x1;
		e->Dx = (x2 - x1) / (double) (y2 - y1);
#else
		e->x1 = (x1 << 16) + 0x8000;
		e->Dx = ((x2 - x1) << 16) / (y2 - y1);
#endif

		e->p = (e + 1)->p = (e + 2)->p = zinfo;

		e->nexte = w->lines[y1].head;
		w->lines[y1].head = e;
	}

	if (y1 < w->ymin)
		w->ymin = y1;

	if (y2 > w->ymax)
		w->ymax = y2;
}


unsigned long
VConstantColor(AWindow *w, int cIndex)
{
	if (w->flags & VPDoubleBuffer) {
		return w->aPixel[cIndex];
	}

	return w->color_to_pixel_map[w->aPixel[cIndex]];
}


VColor   *
VAllocColor(char *name)
{
	return VAllocDepthCueuedColor(name, 0);
}


VColor   *
VAllocDepthCueuedColor(char *name, int flag)
{

	VColor   *p = _VDefaultWorkContext->VColorList, *prev = NULL, **q;

/*
 *  Search for this color among those already allocated.
 */

	while (p != (VColor *) 0) {
		if (strcmp(p->color_name, name) == 0) {
			if ((flag && (p->flags & ColorEnableDepthCueing)) ||
				(flag == 0 && p->flags == 0)) {
				return p;
			}
		}
		prev = p;
		p = p->next;
	}

/*
 *  The color was not in the list; allocate a new list element.
 */

	if (prev == NULL)
		q = &_VDefaultWorkContext->VColorList;
	else
		q = &(prev->next);

	*q = MALLOC(sizeof(VColor));
	(*q)->flags = flag ? ColorEnableDepthCueing : 0;
	(*q)->color_name = STRDUP(name);
	(*q)->cIndex = 0;
	(*q)->next = 0;

	return *q;
}


static int CHECK_I(int i)
{
	static int warn_done = 0;

	if( (i < 0) || (i >= maxpcolors) ){

		if( warn_done == 0 ){
			printf("ERROR: too many colors requested, try reducing color depth cueing.\n");
		}

		printf("ERROR: can't allocate color no. %d of %d allowed.\n",
			i, maxpcolors);

		warn_done++;
		if( i >= AMaxPixelValue || warn_done >= 15 ){
			exit(1);
		}

		i = i % 64;
	}
	return i;
}


static char *errmsg = "ERROR allocating colors, not enough pixel space.\n";
static char *errmsg2 = "ERROR: unable to parse color \"%s\".\n";

#ifdef FIXME_NOT_USED
void
VSetColorTweakProc(AWindow *w, void (*proc) ( /* ??? */ ))
{
	w->colorTweakProc = proc;
}
#endif


static int
pmap(long unsigned int *vec, int n)
{

	static int itbl[] =
	{1, 2, 4, 8};
	register int i, r = 0;

	for (i = 0; i < 4; ++i)
		if (itbl[i] & n)
			r += vec[i];
	return r;
}


static int
_VProcessColor(VWorkContext * cxt, AWindow *w, VColor * vc)
{
	char      realColor[256];
	Display  *display = w->display;
	XColor    xcolor, xcolor2, hcolor;
	double    d;
	int       i, swap = 0;
	unsigned long temp;

/*
 *  First, if we are doing any depth cueing, insure that the haze color has
 *  been converted to RGB values.
 */

	if (cxt->depthCueSteps > 1 && (w->flags & VPDepthCueParsed) == 0) {

		if (w->colorTweakProc) {
			(*w->colorTweakProc) (w, cxt->depthCueColor->color_name,
								  realColor);
		}
		else {
			strcpy(realColor, cxt->depthCueColor->color_name);
		}

		if (XParseColor(display, w->cmap, realColor, &hcolor) == 0) {
			fprintf(stderr, errmsg2, cxt->depthCueColor->color_name);
			return -1;
		}

		if (w->AllocColor(w, w->cmap, &hcolor) == 0) {
			fprintf(stderr, errmsg);
			return -1;
		}

		cxt->depthCueColor->cIndex = cxt->nextPixel;
		w->aPixel[CHECK_I(cxt->nextPixel++)] = hcolor.pixel;
		w->xdepthCueColor = hcolor;
		w->flags |= VPDepthCueParsed;
		swap = 1;
	}

/*
 *  Now parse this color.
 */

	if (w->colorTweakProc) {
		(*w->colorTweakProc) (w, vc->color_name, realColor);
	}
	else {
		strcpy(realColor, vc->color_name);
	}

	if (XParseColor(display, w->cmap, realColor, &xcolor) == 0) {
		fprintf(stderr, errmsg2, vc->color_name);
		return -1;
	}

	if (w->flags & VPMono) {
		vc->cIndex = cxt->nextPixel;
		w->aPixel[CHECK_I(cxt->nextPixel++)]
		= ( xcolor.red * 299L + xcolor.green * 587L + xcolor.blue * 114L )
		/ (1000 * 8192);
	}
	else if( (w->flags & VPDepthCueing)
	&& (cxt->depthCueSteps > 1)
	&& (vc->flags & ColorEnableDepthCueing) ){
		vc->cIndex = cxt->nextPixel;
		hcolor = w->xdepthCueColor;
		for (i = 0; i < cxt->depthCueSteps - 1; ++i) {
			d = (double) i / (double) cxt->depthCueSteps;
			xcolor2.red = (unsigned short) (xcolor.red * (1.0 - d) + hcolor.red * d);
			xcolor2.green = (unsigned short) (xcolor.green * (1.0 - d) + hcolor.green * d);
			xcolor2.blue = (unsigned short) (xcolor.blue * (1.0 - d) + hcolor.blue * d);
			xcolor2.flags = xcolor.flags;
			if (w->AllocColor(w, w->cmap, &xcolor2) == 0) {
				fprintf(stderr, errmsg);
				return -1;
			}
			w->aPixel[CHECK_I(cxt->nextPixel++)] = xcolor2.pixel;
	
		}
	}
	else {
		if (w->AllocColor(w, w->cmap, &xcolor) == 0) {
			fprintf(stderr, errmsg);
			return -1;
		}
		vc->cIndex = cxt->nextPixel;
		w->aPixel[CHECK_I(cxt->nextPixel++)] = xcolor.pixel;
	}

/*
 *  Sorry for the hack, but here goes ...
 *
 *  The background color in the V library is a bit of an orphan.  It is
 *  defined when VBindColors is called.  There are parts of the V library
 *  that assume that w->aPixel[0] is the pixel value of the background color,
 *  but that is not true at this point in time if we just got through parsing
 *  the depth cue color.  By swapping the aPixel[0] and aPixel[1] values
 *  (and updating the corresponding VColor entries that point to them), we
 *  an hack around the problem.
 */

	if (swap) {
		temp = w->aPixel[0];
		w->aPixel[0] = w->aPixel[1];
		w->aPixel[1] = (Color) temp;
		temp = cxt->depthCueColor->cIndex;
		cxt->depthCueColor->cIndex = vc->cIndex;
		vc->cIndex = (Color) temp;
	}

	return 0;
}


int
VBindColors(AWindow *w, char *background)
{

	register int i, j, k, n, c;
	static int parseComplete = 0;
	unsigned int pixel;
	VColor   *p, *bg;
	XColor   *colorSet;
	Display  *display;
	unsigned long planemask[PLANECOUNT * 2];
	unsigned long pixels[1];
	char      realColor[64];

	display = w->display;

	_VDefaultWorkContext->nextPixel = 0;

/*
 *  If this is a color or greyscale pixmap, allocate color cells for each
 *  color, and then set up the Viewport for drawing.
 *
 *  If this is a monochrome monitor, then the aPixel value for this color is
 *  an index into our standard pixmap table rather than being a Pixel value.
 */

	if (w->flags & (VPPixmap | VPFastAnimation | VPMono | VPDoubleBuffer)) {

		bg = VAllocColor(background);

		if (_VProcessColor(_VDefaultWorkContext, w, bg) != 0) {
			return -1;
		}

		for ((i = 0, p = _VDefaultWorkContext->VColorList); p != NULL; i++) {

			if (i > maxpcolors) {
				fprintf(stderr, "Too many colors selected.\n");
				return -1;
			}

			if (_VProcessColor(_VDefaultWorkContext, w, p) != 0) {
				return -1;
			}

			p = p->next;
		}

		w->colors = _VDefaultWorkContext->nextPixel;
		w->set = 0;
		w->pixel = w->aPixel;
		w->mask = AllPlanes;

		return 0;
	}

/*
 *  If we fall through to here, we are doing double buffering using plane
 *  masks.  This is obsolete code, but I'll leave it around for now.
 */

	colorSet = MALLOC(sizeof(XColor) * maxpcolors);

	n = PLANECOUNT;
	c = 1 << n;

	if (XAllocColorCells(display, w->cmap, False, planemask, n * 2, pixels, 1) == 0) {
		fprintf(stderr, "Cannot allocate color cells\n");
		FREE((char *) colorSet);
		return -1;
	}

/*
 *  Parse background color
 */

	if ( /*parseComplete == 0 */ 1) {

		if (w->colorTweakProc) {
			(*w->colorTweakProc) (w, background, realColor);
		}
		else {
			strcpy(realColor, background);
		}

		if (XParseColor(display, w->cmap, realColor, &colorSet[0]) == 0) {
			fprintf(stderr, errmsg2, background);
			FREE(colorSet);
			return -1;
		}

/*
 *  Parse each color defined in the V Color List
 */

		for ((i = 0, p = _VDefaultWorkContext->VColorList); p; i++) {
			if (i > c) {
				fprintf(stderr, "Too many colors selected.\n");
				FREE(colorSet);
				return -1;
			}

			if (w->colorTweakProc) {
				(*w->colorTweakProc) (w, p->color_name, realColor);
			}
			else {
				strcpy(realColor, p->color_name);
			}

			if (XParseColor(display, w->cmap, realColor, &colorSet[i + 1]) == 0) {
				fprintf(stderr, errmsg2, p->color_name);
				FREE(colorSet);
				return -1;
			}
			p->cIndex = i + 1;
			p = p->next;
		}

		parseComplete = 1;
	}

	w->colors = i + 1;

#ifdef DEBUG
	fprintf(stderr, "%d colors defined in the V color list.\n", i);
#endif

/*
 *  PAY ATTENTION!
 *
 *  We will now create a two lists of XColors. Each will expose a particular
 *  drawing buffer (there are two drawing buffers created here).
 *  A drawing is exposed by passing one of these lists to the XSetColors
 *  procedure.
 *  We create a list by iterating through each possible combination of
 *  pixel values, based on the values returned in pixel and planemask.
 *  The pixel value is determined using a function called pmap.  Each pixel
 *  value is assigned the appropriate XColor.
 */

	k = 0;
	for (i = 0; i < w->colors; ++i) {
		pixel = w->aPixel[CHECK_I(i)] = (Color) (pmap(&planemask[0], i) | pixels[0]);
		for (j = 0; j < w->colors; ++j) {
			w->aColor[k] = colorSet[i];
			w->aColor[k++].pixel = pixel | pmap(&planemask[n], j);
		}
	}

	w->aMask = pmap(&planemask[0], (c - 1)) | pixels[0];

	k = 0;
	for (i = 0; i < w->colors; ++i) {
		pixel = w->bPixel[i] = (Color) (pmap(&planemask[n], i) | pixels[0]);
		for (j = 0; j < w->colors; ++j) {
			w->bColor[k] = colorSet[i];
			w->bColor[k++].pixel = pixel | pmap(&planemask[0], j);
		}
	}

	w->bMask = pmap(&planemask[n], (c - 1)) | pixels[0];

	FREE(colorSet);
	return 0;
}


static double sqr(double x)
{
	return x*x;
}


void AlibSetVisibility(double visibility, VColor * haze_color, int steps)
{
	int       i;
	double    k;

	/*
		Release current depth-cueing data:
	*/

	/*
		FIXME: bug: does not release current depthCueColor.
		We need some VFreeColor() counterpart of VAllocColor().
	*/
	// FREE(_VDefaultWorkContext->depthCueColor);
	FREE(_VDefaultWorkContext->visTable);

	/*
		Allocate new depth-cueing data:
	*/

	_VDefaultWorkContext->depthCueColor = haze_color;
	_VDefaultWorkContext->depthCueSteps = steps;
	_VDefaultWorkContext->visibility = visibility;

	_VDefaultWorkContext->visTable = MALLOC(sizeof(double) * steps);

	k = log( 1.0 / (double) steps ) / visibility;

#ifdef DEBUG
	fprintf(stderr, "k = %lg\n", k);
#endif

/*
	Note: the last entry of the table becomes

	visTable[steps-2] = visibility*visibility
*/

	for (i = 0; i < steps - 1; ++i) {
		_VDefaultWorkContext->visTable[i] = sqr(
			log( 1.0 - (double) (i + 1) / (double) steps ) / k
		);
#ifdef DEBUG
		fprintf(stderr, "next threshold is %lg units\n",
			_VDefaultWorkContext->visTable[i]);
#endif
	}

}


int AlibComputePolygonColor(AWindow *w, VPolygon * poly)
{
	VColor   *c;
	VPoint   *p;
	int       i;
	double    d;

/*
 *  First, are we seeing the front or the back of this polygon?
 */

	if (poly->flags & PolyUseBackColor) {
		c = poly->backColor;
	}
	else {
		c = poly->color;
	}

/*
 *  If depth queueing isn't turned on, or this color is not a depth-cueued
 *  color, then simply return the color index.
 */

	if ((w->flags & VPDepthCueing) == 0 ||
		(c->flags & ColorEnableDepthCueing) == 0) {
		return w->pixel[c->cIndex];
	}

/*
 *  Okay, it is a depth cueued color.  Check this polygon's distance against
 *  the different cutoff points for the current world visibility value.  If the
 *  distance is greater than any of the cutoff values, then render the object
 *  as a pure "haze" color.
 */

	if ((i = poly->assignedDepth) > -1) {
		return w->pixel[(i < _VDefaultWorkContext->depthCueSteps - 1) ?
						c->cIndex + i :
						_VDefaultWorkContext->depthCueColor->cIndex];
	}
	p = &poly->vertex[0];
	d = p->x * p->x + p->y * p->y + p->z * p->z;
	for (i = 0; i < _VDefaultWorkContext->depthCueSteps - 1; ++i) {
		if (d < _VDefaultWorkContext->visTable[i]) {
			return w->pixel[c->cIndex + i];
		}
	}
	return w->pixel[_VDefaultWorkContext->depthCueColor->cIndex];
}


void Alib_FreeAll()
{
	VPoly_FreeAll();
}


/* The Alib.c module ends here */
