#ifndef _VOBJECTS_H
#define _VOBJECTS_H

#include "VPoly.h"

typedef struct {
	char     *name;				/* object name */
	VPoint    xaxis, yaxis, zaxis;
	VPoint    center;			/* average of all the object's points */
	double    extent;			/* distance from center to most distant point */
	int       numPolys;			/* polygon count */
	VPolygon **polygon;			/* pointer to array of polygon pointers */
	unsigned short *order;		/* 3-D relative polygon depth ordering */
} VObject;

/*
 *  VObjects and VPolygons are an adequate model for defining simple
 *  objects.  If there are many vertices that are shared among an objects
 *  polygons, though, substantial time may be wasted re-transforming the
 *  same point.  Here we create the notion of a VTerseObject and terse polygon
 *  where all vertices are shared in a common array.
 */

typedef struct {
	short     flags;			/* flag word */
	short     numVtces;			/* vertex count for this polygon */
	VPoint    normal;			/* normal vector plus vertex[0] */
	struct _VSurfaceObject *parent;		/* object that owns this polygon */
	unsigned long *vertex;		/* indices the vertices in this polygon */
	VColor   *color;			/* pointer to color descriptor */
	VColor   *backColor;		/* pointer to back's color (sometimes NULL) */
} VTersePolygon;

typedef struct _VSurfaceObject {
	char     *name;
	VPoint    center;
	double    extent;
	long      numPolys;
	long      numVtces;			/* number of points in "point" array */
	VPoint   *point;			/* pts shared by all VTersePolygons in object */
	VPoint   *tpoint;			/* temporary (transformed) point storage */
	unsigned short *order;
} VTerseObject;

int VComputeObjectAspect (VObject * obj, VPoint * loc);

int VObjectNeedsOrdering (VObject * obj);
/*
	Determine if an object should be ordered. Objects that are not
	made of entirely the same color, or that have colored back-sides
	probably should be ordered.
*/

void VComputePolygonOrdering (VObject * obj);

void VDestroyObject (VObject *obj);

VObject *VCopyObject (VObject *);

VObject * VExtrudeObject(VObject * obj, VPoint * vec);

VObject *VReadObject (FILE * f);

VObject *VReadDepthCueuedObject (FILE * f, int flag);

int VWriteObject (FILE *, VObject *);

void VSetReadObjectScale (VPoint *);

void VComputeObjectExtent (VObject *);

/*
 *  DXF to V-library (obv) file converter
 */

VObject  * VReadDXFObject(FILE *f);

VObject  * VReadDepthCueuedDXFObject(FILE *f, int flag);

#endif /* _VOBJECTS_H */
