/*
	Vdbe -- Basic 2-D drawing routines for X Window with DBE

	This module is a faster alternative to Alib basic 2-D routines for
	systems that have the DBE extension.
*/

#include <Alib.h>
#include <X11/extensions/Xdbe.h>

#define COLLAPSEUNUSEDPOINTS
/*
	Another flag with the same name appears in Alib.c: are the two
	related in some way? To investigate. [U.S.]
*/


void
VdbeExposeBuffer(AWindow *w)
{
	XdbeSwapInfo si;

	si.swap_window = w->win;
	si.swap_action = XdbeUntouched;
	XdbeSwapBuffers(w->display, &si, 1 );
	w->lastPixel = 0xffffffff;
}


void
VdbeDrawLine(AWindow *w, int x1, int y1, int x2, int y2, Color color)
{
	Drawable d;

	d = (Drawable) w->dbeDrawable;
	if (color != w->lastPixel) {
		XSetForeground (w->display, w->gc, color);
		w->lastPixel = color;
	}
	XDrawLine (w->display, d, w->gc, x1, y1, x2, y2);
}


void
VdbeFillRectangle(AWindow *w, Rect *r, Color color)
{
	Drawable d;

	if( color != UnchangedColor && ! IsEmptyRect(r) ){
		d = (Drawable) w->dbeDrawable;
		if (color != w->lastPixel) {
			XSetForeground (w->display, w->gc, color);
			w->lastPixel = color;
		}
		XFillRectangle(w->display, d, w->gc, r->a.x, r->a.y, r->b.x - r->a.x,
			r->b.y - r->a.y);
	}
}


void
VdbeDrawSegments(AWindow *w, Segment * seg, int nseg, Color color)
{
	Drawable  d;

	Display *display = w->display;
	GC gc = w->gc;

	if ( nseg > 1024 ) {
		printf("nseg > 1024; only the first 1024 will be used.\n");
		nseg = 1024;
	}

	d = (Drawable) w->dbeDrawable;

	if (color != w->lastPixel) {
		XSetForeground (display, gc, color);
		w->lastPixel = color;
	}

	XDrawSegments (display, d, gc, seg, nseg);
}


void
VdbeFillPolygon(AWindow *w, Point *pts, int npts, ZInfo *z)
{
#ifdef FIXME_TO_DO
	VPoint   *p;
	XPoint    xpt[VmaxVP], *lastpt;
	register int i, k;
	Drawable  d;
	Color c;
	Display *display = w->display;
	GC xgc = w->gc;

	d = (Drawable) w->dbeDrawable;

	k = 0;
	lastpt = &xpt[0];

	c = AlibComputePolygonColor(w, poly);

	if (c != w->lastPixel) {
		XSetForeground(display, xgc, c);
		w->lastPixel = c;
	}

	for ((i = 0, p = poly->vertex); i < poly->numVtces; (++i, ++p)) {

		if (w->flags & VPPerspective && p->z != 0.0) {
			xpt[k].x = (w->Middl.x + (int) (v->Scale.x * p->x / p->z)) >> 2;
			xpt[k].y = (v->Middl.y - (int) (v->Scale.y * p->y / p->z)) >> 2;
		}
		else {
			xpt[k].x = (v->Middl.x + (int) (v->Scale.x * p->x)) >> 2;
			xpt[k].y = (v->Middl.y - (int) (v->Scale.y * p->y)) >> 2;
		}

#ifdef COLLAPSEUNUSEDPOINTS
		if (k == 0 || !(xpt[k].x == lastpt->x && xpt[k].y == lastpt->y))
			lastpt = &xpt[k++];
#else
		++k;
#endif

	}

	if (k > 0) {
#ifdef COLLAPSEUNUSEDPOINTS
		if (k == 1) {

			XDrawPoint(display, d, xgc, xpt[0].x, xpt[0].y);
		}
		else if (k == 2) {
			XDrawLine(display, d, xgc, xpt[0].x, xpt[0].y,
		xpt[1].x, xpt[1].y);
		}
		else
#endif
			XFillPolygon(display, d, xgc, xpt, k, Convex, CoordModeOrigin);
	}
#endif /* FIXME_TO_DO */
}


void
VdbeSetClipRect(AWindow *w, Rect *r)
{
	XRectangle rect[1];

	if( IsEmptyRect(r) ){
		XSetClipMask(w->display, w->gc, None);
	}
	else {
		rect[0].x = r->a.x;
		rect[0].y = r->a.y;
		rect[0].width = r->b.x - r->a.x;
		rect[0].height = r->b.y - r->a.y;
		XSetClipRectangles(w->display, w->gc, 0, 0, rect, 1, YXBanded);
	}

}


void
VdbeClose(AWindow *w)
{
}


Status
VdbeAllocColor(AWindow *w, Colormap colormap, XColor *c)
{
	Status result;
	result = XAllocColor (w->display, colormap, c);
	return result;
}


/* The Vdbe.c module ends here. */
