/*
 *   DIS/x : An implementation of the IEEE 1278.1 protocol
 *
 *   Copyright (C) 1996, Riley Rainey (rainey@netcom.com)
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of either:
 *
 *   a) the GNU Library General Public License as published by the Free
 *   Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.  A description of the terms and conditions
 *   of the GLPL may be found in the "COPYING.LIB" file.
 *
 *   b) the "Artistic License" which comes with this Kit.  Information
 *   about this license may be found in the "Artistic" file.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License or the Artistic License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Information describing how to contact the author can be found in the
 *   README file.
 */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

#if defined(LIBC_SCCS) && !defined(lint)
/*static char *sccsid = "from: @(#)xdr_mem.c 1.19 87/08/11 Copyr 1984 Sun Micro"; */
/*static char *sccsid = "from: @(#)xdr_mem.c    2.1 88/07/29 4.0 RPCSRC"; */
static char *rcsid = "$Id: xdr_umem.c,v 1.2 2008/03/29 00:26:11 salsi Exp $";

#endif

/*
 * xdr_mem.h, XDR implementation using memory buffers.
 *
 * Copyright (C) 1984, Sun Microsystems, Inc.
 *
 * If you have some data to be interpreted as external data representation
 * or to be converted to external data representation in a memory buffer,
 * then this is the package for you.
 *
 */

#include <strings.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#ifndef WIN32
#include <netinet/in.h>
#endif

static bool_t xdrumem_getint(XDR * xdrs, int *lp);
static bool_t xdrumem_putint(XDR * xdrs, __const int *lp);
static bool_t xdrumem_getbytes(XDR * xdrs, caddr_t addr, register u_int len);
static bool_t xdrumem_putbytes(XDR * xdrs, __const char * addr, register u_int len);
static u_int xdrumem_getpos(__const XDR * xdrs);
static bool_t xdrumem_setpos(XDR * xdrs, u_int pos);
static int32_t *xdrumem_inline(XDR * xdrs, u_int len);
static void xdrumem_destroy(XDR * xdrs);

static struct xdr_ops xdrumem_ops =
{
	xdrumem_getint,
	xdrumem_putint,
	xdrumem_getbytes,
	xdrumem_putbytes,
	xdrumem_getpos,
	xdrumem_setpos,
	xdrumem_inline,
	xdrumem_destroy
};

/*
 * The procedure xdrumem_create initializes a stream descriptor for a
 * memory buffer.  
 */
void
xdrumem_create(register XDR * xdrs, caddr_t addr, u_int size, enum xdr_op op)
{
	xdrs->x_op = op;
	xdrs->x_ops = &xdrumem_ops;
	xdrs->x_private = xdrs->x_base = addr;
	xdrs->x_handy = size;
#ifdef TRACE_XDR
	fprintf(stderr, "<crea xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
}

static void
xdrumem_destroy(XDR * xdrs)
{
#ifdef TRACE_XDR
	fprintf(stderr, ">dest xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
}

static    bool_t
xdrumem_getint(register XDR * xdrs, int *lp)
{
	int      tmp;

/*	fprintf(stderr, ">glon xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u lp=%8.8X\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, lp);
 */	if ((xdrs->x_handy -= sizeof(int)) < 0 ||
	    xdrs->x_private > (xdrs->x_base + 8192)) {
	   if (xdrs->x_private > (xdrs->x_base + 8192))
        	fprintf(stderr,"xdrumem_getint found private overflow\n");
#ifdef TRACE_XDR
		fprintf(stderr, "<getl xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                	xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
	   return (FALSE);
	}
	bcopy(xdrs->x_private, (char *) &tmp, sizeof(int));

	*lp = (int) ntohl((u_int) tmp);
	xdrs->x_private += sizeof(int);
/*	fprintf(stderr, "<glon xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
 */
	return (TRUE);
}

static    bool_t
xdrumem_putint(XDR * xdrs, __const int *lp)
{
	int      tmp;

/*	fprintf(stderr, ">plon xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u lp=%8.8X\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, lp);
 */	if ((xdrs->x_handy -= sizeof(int)) < 0 ||
	    xdrs->x_private > (xdrs->x_base + 8192)) {
	   if (xdrs->x_private > (xdrs->x_base + 8192))
        	fprintf(stderr,"xdrumem_putint found private overflow\n");
#ifdef TRACE_XDR
		fprintf(stderr, "<putl xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                	xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
	   return (FALSE);
	}

	tmp = (int) htonl((u_int) (*lp));
	bcopy((char *) &tmp, xdrs->x_private, sizeof(int));
	xdrs->x_private += sizeof(int);
/*	fprintf(stderr, "<plon xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
 */
	return (TRUE);
}

static    bool_t
xdrumem_getbytes(register XDR * xdrs, caddr_t addr, register u_int len)
{
/*	fprintf(stderr, ">gbyt xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u addr=%8.8X len=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, addr, len);
 */	if ((xdrs->x_handy -= len) < 0 ||
	    xdrs->x_private > (xdrs->x_base + 8192)) {
	   if (xdrs->x_private > (xdrs->x_base + 8192))
        	fprintf(stderr,"xdrumem_getbytes found private overflow\n");
#ifdef TRACE_XDR
		fprintf(stderr, "<getb xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                	xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
	   return (FALSE);
	}
	bcopy(xdrs->x_private, addr, len);
	xdrs->x_private += len;
/*	fprintf(stderr, "<gbyt xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
 */	return (TRUE);
}

static    bool_t
xdrumem_putbytes(register XDR * xdrs, __const char * addr, register u_int len)
{

/*	fprintf(stderr, ">pbyt xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u addr=%8.8X len=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, addr, len);
 */	if ((xdrs->x_handy -= len) < 0 ||
	    xdrs->x_private > (xdrs->x_base + 8192)) {
	   if (xdrs->x_private > (xdrs->x_base + 8192))
        	fprintf(stderr,"xdrumem_putbytes found private overflow\n");
#ifdef TRACE_XDR
		fprintf(stderr, "<putb xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                	xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
	   return (FALSE);
	}
	bcopy(addr, xdrs->x_private, len);
	xdrs->x_private += len;
/*	fprintf(stderr, "<pbyt xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
 */	return (TRUE);
}

static    u_int
xdrumem_getpos(__const XDR * xdrs)
{
	u_int getpos;

	getpos = (u_int) xdrs->x_private - (u_int) xdrs->x_base;
/*	fprintf(stderr, "-gpos xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u pos=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, getpos);
 */	return (getpos);
}

static    bool_t
xdrumem_setpos(register XDR * xdrs, u_int pos)
{
	register caddr_t newaddr = xdrs->x_base + pos;
	register caddr_t lastaddr = xdrs->x_private + xdrs->x_handy;

/*	fprintf(stderr, ">spos xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u pos=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, pos);
        fprintf(stderr, "_spos newaddr=%8.8X lastaddr=%8.8X\n", newaddr, lastaddr);
 */	if ((int) newaddr > (int) lastaddr)
	   return (FALSE);
	xdrs->x_private = newaddr;
	xdrs->x_handy = (int) lastaddr - (int) newaddr;
/*	fprintf(stderr, "<spos xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
 */	return (TRUE);
}

static int32_t *
xdrumem_inline(XDR * xdrs, u_int len)
{
	int32_t     *buf = 0;

/*	fprintf(stderr, ">inli xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u len=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy, len);
 */	if (xdrs->x_private > (xdrs->x_base + 8192)) {
        	fprintf(stderr,"xdrumem_inline found private overflow\n");
#ifdef TRACE_XDR
		fprintf(stderr, "<inli xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                	xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
#endif
	}
	else if (xdrs->x_handy >= len) {
		xdrs->x_handy -= len;
		buf = (int32_t *) xdrs->x_private;
		xdrs->x_private += len;
	}
/*	fprintf(stderr, "<inli xdrs=%8.8X op=%u base=%8.8X private=%8.8X public=%8.8X handy=%u\n",
                xdrs, xdrs->x_op, xdrs->x_base, xdrs->x_private, xdrs->x_public, xdrs->x_handy);
 */	return (buf);
}
