#!/bin/sh
#	Search a directory tree for possible pre-compilation tests of
#	C macros that may be passed to gcc via -D or -U flags, even if
#	they are currently commented-out (yes, this finds some false
#	positives but you can edit those out afterwards).  The output
#	is
#	1. filename_with_relative_path
#	2. first token of the source line
#	3. remaining tokens separated by single spaces
#
#	Output is sorted by $3 to the end of the line,
#	then by $1 through $2, and filtered with uniq
set +xv
grep '^[ 	]*#' `find . -name '*.[ch]'` \
| egrep 'ifdef|ifndef|elif|elifn|if[ \t].*!?defined' \
| sed -e 's/[ \t][ \t]*/\a/g' -e 's/:/\t/' -e 's/\a/\t/' -e 's/\a/\t/' \
| LANG=C sort -k 3.1 -k 1.1,3.1 \
| uniq \
| awk '{printf("%-30.30s %-12.12s %s %s\n", $1, $2, $3, $4)}' \
| sed -e 's/\a/ /g' >macro_tests.txt
