/*
 *  acm : an aerial combat simulator for X
 *  ADF module
 *
 *  Copyright (C) 2007  Umberto Salsi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _adf_h
#define _adf_h

#include "pm.h"

void adf_enable(viewer *u);
void adf_disable(viewer *u);
/*
	Enable/disable the ADF for this particular viewer.
	If u->adf is NULL, instantiate with a pointer to internal data.
*/

void adf_free(viewer *u);
/* Release ADF data u->adf. */

void adf_free_all(void);
/* Release ADF data of ALL the viewsers, so actually freeing the memory. */

/*
	WARNING. In the following functions adf_xxx_inc(u, step) usually
	the value of the step should be +1 (increment) or -1 (decrement)
	simply to indicate the direction of the change.
	Every function internally implements an auto-repeat detection feature
	that speeds-up this otherwise very slow interface. If the auto-repeat
	gets detected, the increment actually applied gets multiplied by a
	factor that ranges from 2 up to 20.
*/

void adf_frq_inc(viewer *u, int step);
/*
	Increment station frequency by step channels.
	ADF channels ranges from 200 up to 415 KHz.
*/

void adf_hdg_inc(viewer *u, int step);
/*
	Increment heading pointer by 'step' DEG.
*/

void adf_panel_draw(viewer * u);
/*
	Draw ADF panel (uses the TEWS panel).
	If no ADF active in this viewer, do nothing.
*/

void adf_draw(viewer * u);
/*
	Draw ADF compass card and OBS pointer.
	If no ADF active in this viewer, do nothing.
*/

#endif
