/*
 *  acm-5.0-ico - AutoPilot System module
 *  Copyright (C) 2007 Umberto Salsi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _aps_h
#define _aps_h

#include "pm.h"

void aps_off(craft * c);

void aps_free(craft * c);
/*
	Release autopilot data.
*/

/*
	Autopilot (AP), aka "hold altitude" and "hold climb speed".
*/
void aps_ap_enable(craft * c);
void aps_ap_disable(craft * c);
_BOOL aps_ap_enabled(craft * c);
void aps_ap_toggle(craft * c);
void aps_ap_set_vs(craft * c, double vs /* m/s */);
void aps_ap_adj_vs(craft * c, double vs /* ft/s */);
/*
	Hold the specified VS. If abs(VS) < 100 fpm, starts AP in hold alt. mode.
*/

double aps_ap_get_delta_elevator(craft * c);
/*
	Returns the computed elevator correction as a number between
	-1.0 and +1.0. Returns 0.0 if AP not enabled.
*/


/*
	AutoCoordinator (AC) - Control the rudder/aileron coordination.
*/
void aps_ac_enable(craft * c);
void aps_ac_disable(craft * c);
void aps_ac_toggle(craft * c);
_BOOL aps_ac_enabled(craft * c);
double aps_ac_get_delta_rudder(craft * c);
/*
	Returns the computed rudder correction as a number between
	-1.0 and +1.0. Returns 0.0 if AC not enabled.
*/

/*
	AutoThrottle (AT) - Hold TAS controlling engine power.
*/
void aps_at_enable(craft * c);
void aps_at_disable(craft * c);
void aps_at_toggle(craft * c);
_BOOL aps_at_enabled(craft * c);

double aps_at_get_velocity(craft * c);
/* Returns current target TAS (ft/s).  */

void aps_at_inc_velocity(craft * c);
void aps_at_dec_velocity(craft * c);
/*
	Increase/decrease the AT target speed by 5 KT.
*/

void aps_bank_max_inc(craft * c);
void aps_bank_max_dec(craft * c);
/*
	Increase/decrease max bank angle. Allowed values are
	15, 20, 25, 30, 35 DEG. Default: 25 DEG.
*/

int aps_bank_max_get(craft * c);
/*
	Return current max bank angle (DEG).
*/

/*
	AutoTurn (AW) - Set a specified turn rate.
*/
void aps_aw_enable(craft * c);
void aps_aw_disable(craft * c);
_BOOL aps_aw_enabled(craft * c);
void aps_aw_set(craft * c, double w);
double aps_aw_get(craft * c);

/*
	AutoNavigation (AN) - Follow VOR outbound radial.
*/
void aps_an_enable(craft * c);
void aps_an_disable(craft * c);
void aps_an_toggle(craft * c);
_BOOL aps_an_enabled(craft * c);

/*
	AutoLanding (AL) - Follow LOC+GS.
*/
void aps_al_enable(craft * c);
void aps_al_disable(craft * c);
void aps_al_toggle(craft * c);
_BOOL aps_al_enabled(craft * c);

/*
	Warning detection. All these functions return TRUE if the corresponding
	automatism reports a warning status: control out of range, can't attain
	the requested value, etc.
*/
_BOOL aps_ap_warn(craft * c);
_BOOL aps_at_warn(craft * c);
_BOOL aps_an_warn(craft * c);
_BOOL aps_al_warn(craft * c);
_BOOL aps_aw_warn(craft * c);
_BOOL aps_ac_warn(craft * c);

void aps_update(craft * c);
/*
	Update autopilot AP, AN, AT, AC, AL internal status.
	Must be called often enough, at leat every 0.1 s.
*/

void aps_free_all(void);
/*
	Release internal data structures.
*/

#endif
