/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991-1998  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _audio_h
#define _audio_h

#include "pm.h"

/*
 *  Pre-defined sound IDs
 */
typedef enum {
	SoundNone = -1,
	SoundEngine,
	SoundCrash,
	SoundGearUp,
	SoundGearDown,
	SoundMissileLaunch,
	SoundCannonFiring,
	SoundExplosion,
	SoundTouchdown,
	SoundStallWarning,
	SoundLockWarning,
	SoundAPGLockAcquired
} SoundID;

_BOOL audio_init(viewer * v, char * display);
/*
	Initialize the sound module on that viewer/server.
	Return FALSE on failure.
*/

void audio_setBackgroundSound(craft * c, double dThtlPercent,
	_BOOL bABFlag, double dDynamicPressure);
/*
	Sets the engine background sound on all the viewers of this plane.
	dThtlPercent: RPM [0.0,1.0]
	bABFlag: after-burner?  (FIXME: actually ignored, anyhow)
	dDynamicPressure: qS / p->wingS (FIXME: actually ignored, anyhow)
	FIXME: actually implemented only for the "poor men" audio driver.
*/

void audio_playSound(craft * c, SoundID id, _BOOL loop);
/*
	Play a sound one time or continously on all the viewers of this plane.
*/

void audio_stopSound(craft * c, SoundID id);
/*
	Stop the execution of the sound started by playSound on all the
	viewers of that plane.
*/

void audio_shutdownAudio(viewer * v);
/*
	Stop execution of any sound on the given viewer and release audio data
	structure.
*/

void audio_update(void);
/*
	To be called continously to update the audio module.
*/

void audio_free_all(void);
/*
	Release internal buffers and data. NOTE: the audio data for every
	viewer MUST be released calling shutdownAudio(viewer).
*/

#endif
