/*
 *  ACM-5.0-ico
 *  Draw utilities module
 *  Copyright (C) 2007  Umberto Salsi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

/*
	This module implements some drawing procedures based on the Vlib
	module. The main difference is that floating point numbers are used
	instead of integer numbers, and the list of segments gets allocated
	dynamically.
*/

#ifndef _DRAW_H
#define _DRAW_H

#include "Vlib.h"


typedef struct _draw_data draw_data;

draw_data * draw_new(void);
void draw_free(draw_data *dd);
/* Allocate/disallocate draw data structure */

void draw_segment(draw_data *dd, double x1, double y1, double x2, double y2);

void draw_rect(draw_data *dd, double x1, double y1, double x2, double y2);

void draw_circle(draw_data *dd, double xo, double yo, double r);

void draw_arc(draw_data *dd, double xo, double yo, double r, double a1, double a2);

void draw_pointer(draw_data *dd, double xo, double yo, double a, double l);
/*
	Draws a pointer of length l with pivot at (xo,yo) and rotated 'a' RAD.
	a=0 means the pointer is oriented toward right, and it rotates
	clockwise on the screen.
*/

void draw_stroke(draw_data *dd, Viewport *v, Color color);
/* Send draw data to the specified viewport using the specified color */

void draw_fill(draw_data *dd, Viewport *v, Color color);
/* FIXME: it does not work yet */

void draw_string_centered(Viewport *v, double xo, double yo, double fh,
	char *s, Color color);

void draw_mark_unchanged(Viewport *v, double xo, double yo, double width,
	double height);
/* FIXME: it does not work yet, apparently a bug in Vlib */

void draw_free_all(void);
/* Release all the internal data of the module */

#endif /* _DRAW_H */
