/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991-1998  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _effects_h
#define _effects_h

#include "pm.h" /* just for the definition of _BOOL... */

void effects_init(void);
/*
	Initializes this module.
*/

void effects_free_all(void);
/*
	Releases internal data structures. Also release data for any mtbl[]
	of type CT_EXPLOSION.

	WARNING. This also removes data that might be used by some craft of
	type CT_EXPLOSION, so all the crafts of this type should be already
	released.
*/

void effects_new_explosion(VPoint * loc, VPoint *vel, double s_meters,
	double dur1, double dur2);
/*
	Add a new "explosion craft" to the list of aircrafts.

	Parameters:
	loc: Location of the explosion (geocentric, m).
	vel: Initial velocity of the explosion (NED, ft/s).
	s_meters: Overall radius of the explosion (m).
	dur1: Dust duration (s).
	dur2: Flame duration (s).
*/

_BOOL effects_explosion_calculations(craft * e);
/*
	Update explosion dynamics.

	Parameters:
	e: The explosion.

	Return: TRUE if it's the time to remove the explosion.
*/

void effects_kill_explosion(craft * e);
/*
	Remove the explosion.

	Parameters:
	e: The explosion.
*/

#endif
