/*
 *  ACM-5.0-ico
 *  Landing gear module
 *  Copyright (C) 2008  Umberto Salsi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _gear_h
#define _gear_h

#include "pm.h"


void gear_up(craft *c);
void gear_down(craft *c);
/*
	Set initial handle and gear state, no animation, no sound
*/

void gear_handle_up(craft *c);
void gear_handle_down(craft *c);
void gear_handle_toggle(craft *c);
/*
 *  Move gear handle, gear animation and sound
 */

void gear_ground_altitude_pitch(craft *c, double *altitude, double *pitch);
/*
 *  Return the rest altitude of the CM over the terrain (ft) and the rest
 *  pitch (RAD) at which the plane can be gently deployed over the runway. The
 *  spring compression are set in order to sustain the total gross weight
 *  empty+fuel+payload.
 *  
 *  ATTENTION. Fuel quantity and payload should be already set before calling
 *  this function.
 */

char * gear_ground_dynamics(craft * c, VPoint *gearF, VPoint *gearM);
/*
 *  Return the total force and moment due to the friction of the wheels on
 *  the ground, components in aircraft XYZ.  Return NULL on success, or the
 *  reason of the crash.
 */

#endif
