/*
 *  acm : an aerial combat simulator for X
 *  Classic instruments module
 *  Copyright (C) 2007  Umberto Salsi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _instruments_h
#define _instruments_h

#include "pm.h"

void inst_enable(viewer * u);
void inst_disable(viewer * u);
void inst_enabled(viewer * u);

void inst_draw(viewer * u);
/*
	Draw the instruments. The data structure u->inst containing the internal
	status of the instruments gets allocated and updated as required.
*/

void inst_attitude_adjust_pitch(viewer * u, double delta);
/*
	The little aircraft frame in front of the attitude indicator can be
	adjusted in pitch by the pilot moving up and down by delta RAD up to
	+/-10 DEG.
*/

void inst_attitude_reset(viewer * u);
/*
	Erect gyro vertical to case orientation.
*/

void inst_altimeter_correction(viewer *u, int delta);
/*
	Set altimeter altitude correction. 'delta' is hundredths of inHg, for
	example if the current altitude correction is 29.92 inHg (the default)
	and delta==-1, the new altitude correction will be 29.91 inHg.
*/

void inst_timer_toggle(viewer *u);
/*
	Start/stop/reset timer.
*/

void inst_free(viewer *u);
/*
	Release the data structure u->inst (if not NULL). Data "released"
	are not actually released from memory, but reused if required.
*/

void inst_free_all(void);
/*
	Release all the internal data structures.
*/

#endif
