/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991,1992  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include "interpolate.h"
#include "mem_manager.h"

/*
 *  interpolate :  determine the value of a function of one variable
 *                 by interpolation.  Interpolation tables are built by
 *                 the ibuild utility.
 */

double
interpolate(ITable * table, double x)
{

	register int i, count = table->count;

	if ( ! (x >= table->minX && x <= table->entry[count - 1].x) ) {
		fprintf(stderr, "interpolate(..., %g): value out of the range [%g,%g]\n",
			x, table->minX, table->entry[count - 1].x);
		abort();
	}

	for (i = 0; i < count; ++i) {
		if (x <= table->entry[i].x) {
			return (table->entry[i].m * x + table->entry[i].b);
		}
	}

	/* should never reach this point */
	abort();
}

ITable *
copyITable (ITable *oldp)
{
	ITable * newp;
	int i;

	if ( oldp == NULL )
		return NULL;

	newp = (ITable *) MALLOC(sizeof(ITable));
	*newp = *oldp;
	newp->entry = (IEntry *) MALLOC(sizeof(IEntry) * (oldp->count));
	for (i=0; i<newp->count; ++i) {
		newp->entry[i] = oldp->entry[i];
	}

	return newp;
}

void
freeITable (ITable *oldp)
{
	if ( oldp != NULL ) {
		FREE( oldp->entry );
		FREE( oldp );
	}
}

