/*
	mem_manager.h - memory allocation routine wrappers
*/

#define MEM_MANAGER_DEBUG


#ifndef MEM_MANAGER_H
#define MEM_MANAGER_H

#ifdef HAVE_STDLIB_H
	#include <stdlib.h>
#else
	#ifdef HAVE_MALLOC_H
		#include <malloc.h>
	#endif
#endif

#ifdef MEM_MANAGER_DEBUG

	void * mem_manager_malloc(int s, char * f, int l);
	void * mem_manager_realloc(void * p, int s, char * f, int l);
	void   mem_manager_free(void *p, char * f, int l);
	char * mem_manager_strdup(char * s, char * f, int l);
	void   mem_manager_report(void);

	#define MALLOC(s)      mem_manager_malloc(s, __FILE__, __LINE__)
	#define FREE(p)        mem_manager_free(p, __FILE__, __LINE__)
	#define REALLOC(p, s)  mem_manager_realloc(p, s, __FILE__, __LINE__)
	#define STRDUP(s)      mem_manager_strdup(s, __FILE__, __LINE__)
	#define MEM_MANAGER_REPORT() mem_manager_report()

#else /* no debug */

	#define MALLOC(s)      mem_manager_malloc(s)
	#define FREE(p)        mem_manager_free(p)
	#define REALLOC(p, s)  mem_manager_realloc(p, s)
	#define STRDUP(s)      mem_manager_strdup(s)
	#define MEM_MANAGER_REPORT()

	void * mem_manager_malloc(int s);
	void * mem_manager_realloc(void * p, int s);
	char * mem_manager_strdup(char *s);
	void   mem_manager_free(void *p);

#endif

// Catch possible direct usages of malloc() realloc() and free() from here on:
#define malloc  invalid_usage_of_malloc_please_use_MALLOC_instead
#define free    invalid_usage_of_free_please_use_FREE_instead
#define realloc invalid_usage_of_realloc_please_use_REALLOC_instead
#ifdef strdup
	#undef strdup
#endif
#define strdup  invalid_usage_of_strdup_please_use_STRDUP_instead

#endif /* MEM_MANAGER_H */
