/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 2007  Umberto Salsi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

/*
 *  2014-12-29 dlc - Refer to CHANGES text file for details of modifications
 */

#include <string.h>
#include "pm.h"
#include "patchlevel.h"
#include "damage.h"

#define EXPIRE_PERIOD 15.0
/* clear current msg after ... (s) */

#define HINT_PERIOD  30.0
/* if no msgs to show, show an hint after ... (s) */

static char *hints[] = {
"Hint: Press the [F2] key to toggle testKeys mode (to learn what keys do) ",
"Hint: Press the [Q] key (up-shifted) twice to exit ACM (self-destruct)   ",
"Hint: Hints that start with a key (e.g., [A] [<] [F3] [Home]) explain it ",
"Hint: Keys [NumLkx] mean the numeric keypad [x] with [NumLock] active    ",
"Hint: [E] redisplays the current hint (shown for 15 seconds)             ",
"Hint: The next hint automatically displays in 45 seconds if convenient   ",
"Hint: [F] displays the next hint (there are dozens)                      ",
"Hint: [J] displays the previous hint (hints wrap around)                 ",
"Hint: [O] displays the first hint (to go back to the beginning)          ",
"Hint: To view the ACM man page, run 'man -l src/acm.man' in a shell      ",
"Hint: If you haven't read the ACM man page, now might be a good time!    ",
"Hint: ACM-7.1-dlc comes with a good manual: study it for best results!   ",
"Hint: Read news and updates at http://dlcusa.net/acm to stay current     ",
"Hint: [d] toggles X key inputs--while disconnected ACM keeps on running  ",
"Hint: [1]-[4] control engine RPM:  1 idle,  2 -~2%,  3 +~2%,  4 full     ",
"Hint: [m] (twice) toggles roll/pitch inputs between mouse and keyboard   ",
"Hint: [K] (twice) starts the joystick calibration procedure (be parked)  ",
"Hint: [U] toggles display of keystick cursor (mouse cursor sync target)  ",
"Hint: [C] cycles through the available HUD/messages colors               ",
"Hint: [B] cycles through the available HUD/messages brightness levels    ",
"Hint: [H] toggles between HUD and classic instruments                    ",
"Hint: [r] cycles through HSI/ADF/radar displays                          ",
"Hint: [R] cycles through HSI and radar displays using DIS beam data      ",
"Hint: [D] toggles debugging display, including GPS and cursor data       ",
"Hint: [t] cycles through timer start/stop/reset                          ",
"Hint: [g] toggles between retracting and extending the landing gear      ",
"Hint: [h] extends flaps one position -- [y] retracts one position        ",
"Hint: [Home] disables all active [AS] systems         [AC AW AP AT AN AL]",
"Hint: [1] and [4] (throttle idle|full) disable all [AS] except [AC]      ",
"Hint: [z] [x] [c] control the rudder when [AC] is disabled (L Center R)  ",
"Hint: [z] [x] [c] do not control ground steering--roll control does that ",
"Hint: [X] toggles auto-coordination (enabling [AW] activates [AC])   [AC]",
"Hint: [|] invokes hold wings-level, cancels any [AW] turn command    [AW]",
"Hint: [<] & [>] invoke standard rate turn L|R (3.0 degrees/second)   [AW]",
"Hint: [,] & [.] invoke L|R turn at 1.5 degrees/second                [AW]",
"Hint: [<] [>] [,] [,] [|] replace each other, only [/] cancels       [AW]",
"Hint: [(] & [)] decrease/increase the max bank setting by 5 degrees  [AW]",
"Hint: [T] toggles auto-throttle to attempt to hold airspeed          [AT]",
"Hint: [2] and [3] increase/decrease target airspeed by 5 knots       [AT]",
"Hint: [Z] toggles hold-altitude, cancels any [AP] command            [AP]",
"Hint: [A] toggles hold-climb/descent, cancels any [AP] command       [AP]",
"Hint: [5] and [6] increase/decrease target rate of climb by ~1 FPM   [AP]",
"Hint: [%] and [^] increase/decrease target rate of climb by ~100 FPM [AP]",
"Hint: [N] toggles auto-navigate via active VOR (activates [AW])      [AN]",
"Hint: [L] toggles ILS auto-landing (activates [AN] and [AP])         [AL]",
"Hint: [Space] cycles through the seven HSI receivers (2 NAV, 5 RNAV)     ",
"Hint: [9] & [0] decrease/increase current NAVAID's frequency (no wrap)   ",
"Hint: [7] & [8] decrease/increase OBS pointer (wraps)                    ",
"Hint: [F3] & [F4] increase/decrease the current RNAV's waypoint radial   ",
"Hint: [F5] & [F6] increase/decrease the current RNAV's waypoint distance ",
"Hint: [+] & [-] zoom in to/out of the outside view by one factor         ",
"Hint: [u] changes the outside view to forward (default)                  ",
"Hint: [o] changes the outside view to left side (no HUD)                 ",
"Hint: [p] changes the outside view to right side (no HUD)                ",
"Hint: [k] changes the outside view to backward (no HUD)                  ",
"Hint: [f] changes the outside view to underneath (no HUD)                ",
"Hint: [e] changes the outside view to overhead (no HUD)                  ",
"Hint: Never exceed 25 degrees of bank for the comfort of your passengers!",
"Hint: [F1] toggles the terminal subsystem (preempts normal key commands} ",
"Hint: The terminal 'help' command displays supported terminal commands   ",
"Hint: The terminal 'create' command does not work at this time           ",
"Hint: The terminal 'set' command does not work at this time              ",
"Hint: The terminal 'v' command displays the aircraft's placard data      ",
"Hint: The placard's Vs0 is stall speed in landing configuration          ",
"Hint: The placard's Vs1 is stall speed in power off cruise configuration ",
"Hint: The placard's Vfe is maximum speed for flaps to be extended        ",
"Hint: The placard's Vno is maximum speed for normal cruising in any air  ",
"Hint: The placard's Vne is never exceed speed (ideal conditions cruise)  ",
"Hint: The placard's alpha stall is the angle of attack for stalling      ",
"Hint: The placard's MTOW is maximum take-off weight--don't overload!     ",
"Hint: The placard's total mass is unclear and confusing--ignore it       ",
"Hint: Exceeding the aircraft's maximums can induce structural failure!   ",
"Hint: Your plane lacks the speed brakes? use the flaps instead!          ",
"Hint: One nautical mile corresponds to about 2 kilometers                ",
"Hint: One knot means one nautical mile per hour                          ",
"Hint: Three thousand feet is almost exactly one kilometer                ",
"Hint: There are old pilots and bold pilots, but no old bold pilots.      "
};

#define BUF_SIZE 100

typedef struct _prompt_data {
	struct _prompt_data *next;

	_BOOL have_msg;
	double expire;
	char buf[BUF_SIZE];

	double last_msg_shown_at;
} prompt_data;

static prompt_data *free_list = NULL;


void prompt_free(viewer *u)
{
	prompt_data *pd;

	if( u->prompt == NULL )
		return;
	
	pd = u->prompt;

	pd->next = free_list;
	free_list = pd;
	u->prompt = NULL;
}


void prompt_free_all()
{
	prompt_data *pd;

	while( free_list != NULL ){
		pd = free_list;
		free_list = pd->next;
		FREE(pd);
	}
}


void prompt_viewer_print(viewer *u, char *s)
{
	prompt_data *pd;

	if( u->prompt == NULL ){
		if( free_list == NULL ){
			pd = MALLOC( sizeof(prompt_data) );
		} else {
			pd = free_list;
			free_list = pd->next;
		}
		pd->next = NULL;
		pd->have_msg = FALSE;
		pd->expire = 0.0;
		pd->buf[0] = '\0';
		pd->last_msg_shown_at = -HINT_PERIOD; /* next msg shown immediately */
		u->prompt = pd;
	} else {
		pd = u->prompt;
	}

	pd->expire = curTime + EXPIRE_PERIOD;
	strncpy(pd->buf, s, BUF_SIZE);
	pd->buf[BUF_SIZE-1] = '\0';
	pd->have_msg = TRUE;
}


void prompt_viewer_print_hint(viewer *u, int k)
{
	switch (k) {
	    case 1:	/* next */
		u->hint++; if (u->hint == sizeof(hints) / sizeof(char *)) u->hint = 0;
		break;
	    case 2:	/* previous */
		u->hint--; if (u->hint < 0) u->hint = sizeof(hints) / sizeof(char *) - 1;
		break;
	    case 3:	/* first */
	        u->hint = 0;
		break;
	    default:	/* current */
		break;
	}
	prompt_viewer_print(u, hints[u->hint]);
}


void prompt_craft_print(craft *c, char *s)
{
	viewer *v;

	v = vl_head;

	while( v != NULL ){
		if( v->c == c )
			prompt_viewer_print(v, s);
		v = v->next;
	}
}


void prompt_broadcast_print(char *s)
{
	viewer *v;

	v = vl_head;

	while( v != NULL ){
		prompt_viewer_print(v, s);
		v = v->next;
	}
}


static void draw_string(Viewport *v, prompt_data *pd)
{
	double fh, fw, w, k;
	int width, height, l, x, y;

	width = RectWidth(v->rect);
	height = RectHeight(v->rect);
	fh = ceil(height / 30.0);
	fw = VFontWidthPixels(v, (int) (fh+0.5));
	l = strlen(pd->buf);
	k = l * fw / (0.95*width);
	if( k >= 1.0 ){
		fh = floor(fh/k);
		fw = VFontWidthPixels(v, (int) (fh+0.5));
	}
	w = fw * l;

	AlibSetClipRect(v->w, &v->rect);
	x = v->rect.a.x + width/2 - w/2.0;
	y = v->rect.b.y - fh;
	VDrawStrokeString(v, x, y,
		pd->buf, l, fh, HUDPixel);
}


void prompt_draw(viewer *u)
{
	prompt_data *pd;

	if( u->prompt == NULL ){
		prompt_viewer_print(u, "Welcome to ACM-" REVISION_STRING
			"     Watch this space for hints");
		return;
	}
	
	pd = u->prompt;

	/*
		Display hints only in stright, level, at constant speed flight
	*/
	if(
		fabs(u->c->Cg.z)*60.0 > 90.0 /* ft/min */
	|| fabs(u->c->curRoll) > DEGtoRAD(5.0)
	|| fabs(u->c->G.x) > 0.1
	)
		pd->last_msg_shown_at = curTime + HINT_PERIOD;


	if( pd->have_msg ){
		/*
			Show current msg:
		*/

		if( curTime > pd->expire ){
			pd->have_msg = FALSE;
			pd->last_msg_shown_at = curTime;
			return;
		}

		draw_string(u->v, pd);
	
	} else if( pd->last_msg_shown_at + HINT_PERIOD < curTime ){
		/*
			Show current hint and increment the hint array index:
		*/

		prompt_viewer_print_hint(u, 0);
		u->hint++; if (u->hint == sizeof(hints) / sizeof(char *)) u->hint = 0;
	}
}

/* End of the prompt module. */
