#include <string.h>

#include "adf.h"
#include "audio.h"
#include "hsi.h"
#include "hud.h"
#include "instruments.h"
#include "magnetic_compass.h"
#include "mem_manager.h"
#include "pm.h"
#include "prompt.h"
#include "render.h"
#include "terminal.h"

#include "viewer.h"


viewer * viewer_new(craft * c)
{
	viewer *v;

	v = (viewer *) MALLOC(sizeof(viewer));
	memset(v, 0, sizeof(viewer));

	v->vl_next = vl_head;
	vl_head = v;

	v->next = c->vl;
	c->vl = v;

	v->viewer_state = ViewerStateNormal;
	v->c = c;
	v->watchedCraft = NULL;
	v->hasFocus = FALSE;
	v->hasComm = TRUE;

	/* FIXME: to-do fields
	v->w = ...;
	v->v = ...;
	v->dpy = ...;
	v->win = ...;
	v->gc = ...;
	v->cn = ...;
	v->protocolsAtom = ...;
	v->deleteWindowAtom = ...;
	v->closedownAtom = ...;

	v->hud_yCenter = ...
	*/

	/*
		All the Rect, drawnItem and scaleControl fields are in charge
		of the layout manager (se "windows" module).
	*/

	v->zoom = 16;
	v->hud_mode = FALSE;

	/* FIXME: to-do
	v->xscaleFactor = ...
	v->yscaleFactor = ...
	v->z =
	v->rz = ...
	v->radarImage = ...
	v->radarImageCount = ...
	*/
	v->browseBase = 0;
	v->browseSelectedItem = -1;
	v->browseClickTime = 0;

	/* Set v->viewDirection and v->viewUp: */
	setOutsideView(c, v, VIEW_FORWARD);

	v->hud = NULL;
	v->magnetic_compass = NULL;
	v->hsi = NULL;
	v->adf = NULL;
	v->audio = NULL;
	v->inst = NULL;
	v->terminal = NULL;
	v->prompt = NULL;

	return v;
}


void viewer_free(viewer *v)
{
	viewer *cur;
	craft  *c;

	/*
		Release attached services:
	*/

	audio_shutdownAudio(v);
	hud_free(v);
	magnetic_compass_free(v);
	adf_free(v);
	hsi_free(v);
	inst_free(v);
	terminal_free(v);
	prompt_free(v);

	/*
		Close the window
	*/

	VCloseViewport(v->v);
	AlibClose(v->w);
	XCloseDisplay(v->w->display);

	/*
		Remove this viewer from the craft's list of viewers:
	*/

	c = v->c;
	
	if( c->vl == v ){
		c->vl = v->next;
	} else {
		cur = c->vl;
		while( cur->next != v )
			cur = cur->next;
		cur->next = v->next;
	}

	/*
		Remove viewer from the list of viewers:
	*/

	if( vl_head == v ){
		vl_head = v->vl_next;
	} else {
		cur = vl_head;
		while( cur->vl_next != v )
			cur = cur->vl_next;
		cur->vl_next = v->vl_next;
	}

	FREE(v);

}
