/*
 *  acm : an aerial combat simulator for X
 *  Copyright (C) 1991-1998  Riley Rainey
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 dated June, 1991.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program;  if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
 */

#ifndef _weapon_h
#define _weapon_h

#include "pm.h"

/* Index of the weapon into the wtbl[] table: */
#define WEAPON_NULL     0     /* null weapon, or no weapon selected */
#define WEAPON_M61A1    1     /* M-61A1 Vulcan 20mm cannon */
#define WEAPON_AIM9M    2     /* Sidewinder missile */
#define WEAPON_AIM120   3     /* AIM-120 missile */
#define WEAPON_MK82     4     /* Mark 82 (510 lb) gravity bomb */
//#define WEAPON_AGM65    5      /* FIXME: not used */

typedef struct
/*
	Defines call-back procedures for all the standard actions of a weapon.
*/
{
	int  (*select) (struct _craft *);
	/*
		Weapon select procedure invoked by pilot.
		Return: TRUE if there is actually a weapon of this type
		available and functional.
	*/

	int  (*update) (struct _craft *);
	/*
		Per tick update procedure that implements weapon dynamics.
		Return: FIXME.
	*/

	int  (*display) (struct _craft *, craftType *, viewer *, int, int);
	/*
		Display (that is HUD) update procedure.
		Return: TRUE if there is a reasonable chance of scoring a kill.
	*/

	int  (*firePress) (struct _craft *);
	/*
		Fire button pressed procedure.
		Return: FIXME.
	*/

	int  (*fireRelease) (struct _craft *);
	/*
		Fire button released.
		Return: FIXME.
	*/

	craftType *w;
	/*
		Description of this weapon.
	*/

} WeaponDesc;


void weapon_init(void);
/*
	Initialize the module.
*/

int weapon_name_to_id(char * name);
/*
	Return the weapon number WK_* given its name.
	Return a negative value if name unknown.
*/


char * weapon_id_to_name(int id);
/*
	Return the weapon name given its id WK_*.
	Return NULL if the id is invalid.
*/

int ceaseFireWeapon(craft * c);

int countOrdinance(craft * c, int id);
/*
 * Count a particular type of ordinance on an aircraft, where id is
 * one of the constants WK_* (see pm.h).
 */

int doWeaponDisplay(craft * c, viewer * u, int fpm_x, int fpm_y);
/*
 *  Draw HUD data specific of the currently selected weapon.
 *  fpm_x,fpm_y is the current position of the HUD FPM (used by the
 *  ccip module).
 *  Return a nonzero value if have a reasonable chance of scoring a kill.
 */

int doWeaponUpdate(craft * c);

int fireWeapon(craft * c);
/*
	Fire the weapon currently selected.
	Returns 0 if no weapon is currently available.
*/

int readyStation(craft * c, int id);
/*
 *  Return the index of the first station with a particular type of ordinance
 *  ready for fire.
 *  id is one of the constants WK_* (see pm.h).
 */

int selectNamedWeapon(craft * c, int id);
/*
	Select the given weapon number id (see constants WK_* in pm.h).
	Returns 0 if the selection cannot be operated.
*/

int selectWeapon(craft * c);
/*
	Select next available type of weapon.
	Returns 0 if the selection cannot be operated.
*/

#endif
